%% Drilling simulation
close all;
clear all;
warning('off','all');
str = fileparts(which('drilling_case_1.m'));
cd(str);
run '../path_config.m'
%% User interface
% Instructions:
% Set flag = 1 to plot the respective quantity
% Set flag = 0 to not plot the respective quantity

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
plt.flags = [];
plt.flags(drill_plot_enums.drill_state_p) =                                 1;% Drillstring pressure p(z,t)
plt.flags(drill_plot_enums.drill_state_q) =                                 1;% Drillstring flow q(z,t)
plt.flags(drill_plot_enums.bilat_obs_est_hat_p) =                           0;% Bilateral observer estimate \hat{p}(z,t)
plt.flags(drill_plot_enums.bilat_obs_est_hat_q) =                           0;% Bilateral observer estimate \hat{q}(z,t)
plt.flags(drill_plot_enums.unilat_obs_est_hat_p) =                          0;% Unilateral observer estimate \hat{p}(z,t)
plt.flags(drill_plot_enums.unilat_obs_est_hat_q) =                          0;% Unilateral observer estimate \hat{q}(z,t)
plt.flags(drill_plot_enums.bilat_obs_est_err_tild_p) =                      0;% Bilateral observer estimate error \tilde{p}(z,t)
plt.flags(drill_plot_enums.bilat_obs_est_err_tild_q) =                      0;% Bilateral observer estimate error \tilde{q}(z,t)
plt.flags(drill_plot_enums.unilat_obs_est_err_tild_p) =                     0;% Unilateral observer estimate error \tilde{p}(z,t)
plt.flags(drill_plot_enums.unilat_obs_est_err_tild_q) =                     0;% Unilateral observer estimate error \tilde{q}(z,t)
plt.flags(drill_plot_enums.obs_err_norm_p) =                                0;% Observer Euclidean error norms ||p||(t)
plt.flags(drill_plot_enums.obs_err_norm_q) =                                0;% Observer Euclidean error norms ||q||(t)
plt.flags(drill_plot_enums.obs_downhole_est_p) =                            0;% Observer downhole estimates \hat{p}(0,t)
plt.flags(drill_plot_enums.obs_downhole_est_q) =                            0;% Observer downhole estimates \hat{q}(0,t)
plt.flags(drill_plot_enums.obs_downhole_err_tild_p) =                       0;% Observer downhole estimate errors \tilde{p}(0,t)
plt.flags(drill_plot_enums.obs_downhole_err_tild_q) =                       0;% Observer downhole estimate errors \tilde{q}(0,t)
plt.flags(drill_plot_enums.obs_topside_est_p) =                             0;% Observer topside estimates \hat{p}(l,t)
plt.flags(drill_plot_enums.obs_topside_est_q) =                             0;% Observer topside estimates \hat{q}(l,t)
plt.flags(drill_plot_enums.obs_topside_err_tild_p) =                        0;% Observer topside estimate errors \tilde{p}(l,t)
plt.flags(drill_plot_enums.obs_topside_err_tild_q) =                        0;% Observer topside estimate errors \tilde{q}(l,t)
plt.flags(drill_plot_enums.bilat_obs_gain_P_pp) =                           0;% Bilateral observer gain P^{++}(x)
plt.flags(drill_plot_enums.bilat_obs_gain_P_pm) =                           0;% Bilateral observer gain P^{+-}(x)
plt.flags(drill_plot_enums.bilat_obs_gain_P_mp) =                           0;% Bilateral observer gain P^{-+}(x)
plt.flags(drill_plot_enums.bilat_obs_gain_P_mm) =                           0;% Bilateral observer gain P^{--}(x)
plt.flags(drill_plot_enums.unilat_obs_gain_P_p) =                           0;% Unilateral observer gain P^{+}(x)
plt.flags(drill_plot_enums.unilat_obs_gain_P_m) =                           0;% Unilateral observer gain P^{-}(x)
plt.flags(drill_plot_enums.kernel_M11) =                                    0;% M11 kernel equation solution
plt.flags(drill_plot_enums.kernel_M12) =                                    0;% M12 kernel equation solution
plt.flags(drill_plot_enums.kernel_M21) =                                    0;% M21 kernel equation solution
plt.flags(drill_plot_enums.kernel_M22) =                                    0;% M22 kernel equation solution
plt.flags(drill_plot_enums.kernel_N11) =                                    0;% N11 kernel equation solution
plt.flags(drill_plot_enums.kernel_N12) =                                    0;% N12 kernel equation solution
plt.flags(drill_plot_enums.kernel_N21) =                                    0;% N21 kernel equation solution
plt.flags(drill_plot_enums.kernel_N22) =                                    0;% N22 kernel equation solution
plt.flags(drill_plot_enums.kernel_K21) =                                    0;% K21 kernel equation solution
plt.flags(drill_plot_enums.kernel_M) =                                      0;% M kernel equation solution
plt.flags(drill_plot_enums.kernel_N) =                                      0;% N kernel equation solution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Simulation parameters
sys.N_g = 100;
sys.xspan = linspace(0, 1, sys.N_g);
sys.N_g_sim = 2*sys.N_g;
sys.xspan_sim = linspace(0, 1, sys.N_g_sim);
sys.Delta_sim = 1/(sys.N_g_sim - 1);
sys.simTime = 30;
sys.h = 0.001;
sys.Tspan = 0:sys.h:sys.simTime;
sys.N_sim = sys.simTime/sys.h + 1;

%% Drilling system parameters

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Drilling system
%
% z \in [0,l], t \in [0, \infty)
%
% p_t(z,t) = -(\beta/A_1)q_z(z,t)
% q_t(z,t) = -(A_1/\rho) p_z(z,t) - (F_1/\rho)q(z,t) - A_1g
%
% with boundary conditions
% 
% q(0,t)   = J(p_r(t) - p(0,t)) + q_bit(t)
% p(l,t)   = p_l(t)
%
% and initial conditions
%
% q(z,0) = q0(z)
% p(z,0) = p0(z)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

drill.kick_pres = 500*10^5;%Extra pressure from kick[Pa]
drill.kick_time = 5;%Time at which kick occurs[s]
drill.res_pres_init = 400*10^5;%Initial reservoir pressure
drill.distDelay = 10; % Time for correct reservoir pressure to be estimated after kick

drill.beta = 7317*10^5;%Bulk modulus of mud [Pa]
drill.rho = 1250;%Density of mud [kg/m^3]'
drill.A_1 = 0.024;%Cross sectional area of annulus [m^2]
drill.F_1 = 600;%Friction factor [kg/(m^3*s)]
drill.g = 9.81;%Acceleration of gravity [m/s^2];
drill.l = 3000;%Depth of well [m]
drill.p_sp = drill.res_pres_init;%Pressure setpoint [Pa]
drill.p_r = @(t)(drill.res_pres_init + drill.kick_pres*heaviside(t-drill.kick_time));% Reservoir pressure 
drill.p_l = @(t)(drill.p_sp - drill.rho*drill.g*drill.l);% Topside pressure control
drill.q_bit = @(t)(1/60);%Downhole flow control
drill.J = 1.1*10^(-8);%Production index
drill.q0 = @(z)(1/60);
drill.p0 = @(z)(drill.p_sp - drill.rho*drill.g*z);

sys.zspan_sim = drill.l*sys.xspan_sim;
%% Plotting options
set(groot, 'DefaultTextInterpreter', 'Latex');
set(groot, 'DefaultLegendInterpreter', 'Latex');
sys.t_skip = 6;
sys.x_skip = 1;
sys.t_start = 0;                                                           %Time when plotting should start
sys.t_start_grid = sys.t_start/sys.h+1;                                    %Time grid point where plotting should start
plt.surf_view_time = [120, 20];
plt.surf_view_kernel = [-65, 20];

plt.axis_set_p_surf =  [0 drill.l sys.t_start sys.simTime 0 11*10^7];
plt.axis_set_q_surf =  [0 drill.l sys.t_start sys.simTime 0 1.5];
plt.axis_set_p_err_surf =  [0 drill.l sys.t_start sys.simTime -10^6 25*10^6];
plt.axis_set_q_err_surf =  [0 drill.l sys.t_start sys.simTime -0.1 0.5];
plt.axis_set_p_plot =  [sys.t_start sys.simTime -10^7 11*10^7];
plt.axis_set_q_plot =  [sys.t_start sys.simTime 0 1.5];
plt.axis_set_p_err_plot =  [sys.t_start sys.simTime 0 2*10^7];
plt.axis_set_q_err_plot =  [sys.t_start sys.simTime -0.1 0.4];
plt.axis_set_obs_gains = [0 1 -0.35 0.1];
plt.axis_set_kernel = [0 1 0 1 -0.7 0.25];

%Positions of plot |x|y|
plt.positions = []; 
plt.graphics = groot;
plt.x0_right = plt.graphics.MonitorPositions(1,1);
plt.y0_right = plt.graphics.MonitorPositions(1,2);
if(size(plt.graphics.MonitorPositions, 1) == 1) %1 monitor
    plt.x0_left = plt.x0_right;
    plt.y0_left = plt.y0_right;
else                                            %2 or more monitors
    plt.x0_left = plt.graphics.MonitorPositions(2,1);
    plt.y0_left = plt.graphics.MonitorPositions(2,2);
end
plt.screen_width = plt.graphics.ScreenSize(3);
plt.screen_height = plt.graphics.ScreenSize(4);

plt.positions(drill_plot_enums.drill_state_q, :) =             [plt.x0_left,                             plt.y0_left + (4/5)*plt.screen_height ];% Drillstring pressure p(z,t)
plt.positions(drill_plot_enums.drill_state_p, :) =             [plt.x0_left + (1/4)*plt.screen_width,    plt.y0_left + (4/5)*plt.screen_height ];% Drillstring flow q(x,t) 
plt.positions(drill_plot_enums.bilat_obs_est_hat_p, :) =       [plt.x0_left,                             plt.y0_left + (3/5)*plt.screen_height ];% Bilateral observer estimate \hat{p}(z,t)
plt.positions(drill_plot_enums.bilat_obs_est_hat_q, :) =       [plt.x0_left,                             plt.y0_left + (2/5)*plt.screen_height ];% Bilateral observer estimate \hat{q}(z,t)
plt.positions(drill_plot_enums.unilat_obs_est_hat_p, :) =      [plt.x0_left + (1/4)*plt.screen_width,    plt.y0_left + (3/5)*plt.screen_height ];% Unilateral observer estimate \hat{p}(z,t)
plt.positions(drill_plot_enums.unilat_obs_est_hat_q, :) =      [plt.x0_left + (1/4)*plt.screen_width,    plt.y0_left + (2/5)*plt.screen_height ];% Unilateral observer estimate \hat{q}(z,t)
plt.positions(drill_plot_enums.bilat_obs_est_err_tild_p, :) =  [plt.x0_left + (2/4)*plt.screen_width,    plt.y0_left + (3/5)*plt.screen_height ];% Bilateral observer estimate error \tilde{p}(z,t)
plt.positions(drill_plot_enums.bilat_obs_est_err_tild_q, :) =  [plt.x0_left + (2/4)*plt.screen_width,    plt.y0_left + (2/5)*plt.screen_height ];% Bilateral observer estimate error \tilde{q}(z,t)
plt.positions(drill_plot_enums.unilat_obs_est_err_tild_p, :) = [plt.x0_left + (3/4)*plt.screen_width,    plt.y0_left + (3/5)*plt.screen_height ];% Unilateral observer estimate error \tilde{p}(z,t)
plt.positions(drill_plot_enums.unilat_obs_est_err_tild_q, :) = [plt.x0_left + (3/4)*plt.screen_width,    plt.y0_left + (2/5)*plt.screen_height ];% Unilateral observer estimate error \tilde{q}(z,t)
plt.positions(drill_plot_enums.obs_err_norm_p, :) =            [plt.x0_left + (2/4)*plt.screen_width,    plt.y0_left + (4/5)*plt.screen_height ];% Observer Euclidean error norms ||p||(t)
plt.positions(drill_plot_enums.obs_err_norm_q, :) =            [plt.x0_left + (3/4)*plt.screen_width,    plt.y0_left + (4/5)*plt.screen_height ];% Observer Euclidean error norms ||q||(t)
plt.positions(drill_plot_enums.obs_downhole_est_p, :) =        [plt.x0_left,                             plt.y0_left + (1/5)*plt.screen_height ];% Observer downhole estimates \hat{p}(0,t)
plt.positions(drill_plot_enums.obs_downhole_est_q, :) =        [plt.x0_left + (1/4)*plt.screen_width,    plt.y0_left + (1/5)*plt.screen_height ];% Observer downhole estimates \hat{q}(0,t)
plt.positions(drill_plot_enums.obs_downhole_err_tild_p, :) =   [plt.x0_left + (2/4)*plt.screen_width,    plt.y0_left + (1/5)*plt.screen_height ];% Observer downhole estimate errors \tilde{p}(0,t)
plt.positions(drill_plot_enums.obs_downhole_err_tild_q, :) =   [plt.x0_left + (3/4)*plt.screen_width,    plt.y0_left + (1/5)*plt.screen_height ];% Observer downhole estimate errors \tilde{q}(0,t)
plt.positions(drill_plot_enums.obs_topside_est_p, :) =         [plt.x0_left,                             plt.y0_left                           ];% Observer topside estimates \hat{p}(l,t)
plt.positions(drill_plot_enums.obs_topside_est_q, :) =         [plt.x0_left + (1/4)*plt.screen_width,    plt.y0_left                           ];% Observer topside estimates \hat{q}(l,t)
plt.positions(drill_plot_enums.obs_topside_err_tild_p, :) =    [plt.x0_left + (2/4)*plt.screen_width,    plt.y0_left                           ];% Observer topside estimate errors \tilde{p}(l,t)
plt.positions(drill_plot_enums.obs_topside_err_tild_q, :) =    [plt.x0_left + (3/4)*plt.screen_width,    plt.y0_left                           ];% Observer topside estimate errors \tilde{q}(l,t)
plt.positions(drill_plot_enums.bilat_obs_gain_P_pp, :) =       [plt.x0_right,                          plt.y0_right                            ];% Bilateral observer gain P^{++}(x)
plt.positions(drill_plot_enums.bilat_obs_gain_P_pm, :) =       [plt.x0_right + (1/5)*plt.screen_width, plt.y0_right                            ];% Bilateral observer gain P^{+-}(x)
plt.positions(drill_plot_enums.bilat_obs_gain_P_mp, :) =       [plt.x0_right + (2/5)*plt.screen_width, plt.y0_right                            ];% Bilateral observer gain P^{-+}(x)
plt.positions(drill_plot_enums.bilat_obs_gain_P_mm, :) =       [plt.x0_right + (3/5)*plt.screen_width, plt.y0_right                            ];% Bilateral observer gain P^{--}(x)
plt.positions(drill_plot_enums.unilat_obs_gain_P_p, :) =       [plt.x0_right + (4/5)*plt.screen_width, plt.y0_right + (1/4)*plt.screen_height  ];% Unilateral observer gain P^+(x)
plt.positions(drill_plot_enums.unilat_obs_gain_P_m, :) =       [plt.x0_right + (4/5)*plt.screen_width, plt.y0_right                            ];% Unilateral observer gain P^-(x)
plt.positions(drill_plot_enums.kernel_M11, :) =                [plt.x0_right,                          plt.y0_right + (3/4)*plt.screen_height  ];% M11 kernel equation solution
plt.positions(drill_plot_enums.kernel_M12, :) =                [plt.x0_right + (1/5)*plt.screen_width, plt.y0_right + (3/4)*plt.screen_height  ];% M12 kernel equation solution
plt.positions(drill_plot_enums.kernel_M21, :) =                [plt.x0_right,                          plt.y0_right + (2/4)*plt.screen_height  ];% M21 kernel equation solution
plt.positions(drill_plot_enums.kernel_M22, :) =                [plt.x0_right + (1/5)*plt.screen_width, plt.y0_right + (2/4)*plt.screen_height  ];% M22 kernel equation solution
plt.positions(drill_plot_enums.kernel_N11, :) =                [plt.x0_right + (2/5)*plt.screen_width, plt.y0_right + (3/4)*plt.screen_height  ];% N11 kernel equation solution
plt.positions(drill_plot_enums.kernel_N12, :) =                [plt.x0_right + (3/5)*plt.screen_width, plt.y0_right + (3/4)*plt.screen_height  ];% N12 kernel equation solution
plt.positions(drill_plot_enums.kernel_N21, :) =                [plt.x0_right + (2/5)*plt.screen_width, plt.y0_right + (2/4)*plt.screen_height  ];% N21 kernel equation solution
plt.positions(drill_plot_enums.kernel_N22, :) =                [plt.x0_right + (3/5)*plt.screen_width, plt.y0_right + (2/4)*plt.screen_height  ];% N22 kernel equation solution
plt.positions(drill_plot_enums.kernel_K21, :) =                [plt.x0_right + (4/5)*plt.screen_width, plt.y0_right + (3/4)*plt.screen_height  ];% K21 kernel equation solution
plt.positions(drill_plot_enums.kernel_M, :)   =                [plt.x0_right,                          plt.y0_right + (1/4)*plt.screen_height  ];% M kernel equation solution
plt.positions(drill_plot_enums.kernel_N, :)   =                [plt.x0_right + (1/5)*plt.screen_width, plt.y0_right + (1/4)*plt.screen_height  ];% N kernel equation solution

%Width and height of plots |width|height|
plt.size = [];   
plt.size(drill_plot_enums.drill_state_q, :) =             [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Drillstring pressure p(z,t)
plt.size(drill_plot_enums.drill_state_p, :) =             [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Drillstring flow q(x,t) 
plt.size(drill_plot_enums.bilat_obs_est_hat_p, :) =       [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Bilateral observer estimate \hat{p}(z,t)
plt.size(drill_plot_enums.bilat_obs_est_hat_q, :) =       [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Bilateral observer estimate \hat{q}(z,t)
plt.size(drill_plot_enums.unilat_obs_est_hat_p, :) =      [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Unilateral observer estimate \hat{p}(z,t)
plt.size(drill_plot_enums.unilat_obs_est_hat_q, :) =      [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Unilateral observer estimate \hat{q}(z,t)
plt.size(drill_plot_enums.bilat_obs_est_err_tild_p, :) =  [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Bilateral observer estimate error \tilde{p}(z,t)
plt.size(drill_plot_enums.bilat_obs_est_err_tild_q, :) =  [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Bilateral observer estimate error \tilde{q}(z,t)
plt.size(drill_plot_enums.unilat_obs_est_err_tild_p, :) = [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Unilateral observer estimate error \tilde{p}(z,t)
plt.size(drill_plot_enums.unilat_obs_est_err_tild_q, :) = [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Unilateral observer estimate error \tilde{q}(z,t)
plt.size(drill_plot_enums.obs_err_norm_p, :) =            [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Observer Euclidean error norms ||p||(t)
plt.size(drill_plot_enums.obs_err_norm_q, :) =            [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Observer Euclidean error norms ||q||(t)
plt.size(drill_plot_enums.obs_downhole_est_p, :) =        [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Observer downhole estimates \hat{p}(0,t)
plt.size(drill_plot_enums.obs_downhole_est_q, :) =        [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Observer downhole estimates \hat{q}(0,t)
plt.size(drill_plot_enums.obs_downhole_err_tild_p, :) =   [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Observer downhole estimate errors \tilde{p}(0,t)
plt.size(drill_plot_enums.obs_downhole_err_tild_q, :) =   [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Observer downhole estimate errors \tilde{q}(0,t)
plt.size(drill_plot_enums.obs_topside_est_p, :) =         [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Observer topside estimates \hat{p}(l,t)
plt.size(drill_plot_enums.obs_topside_est_q, :) =         [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Observer topside estimates \hat{q}(l,t)
plt.size(drill_plot_enums.obs_topside_err_tild_p, :) =    [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Observer topside estimate errors \tilde{p}(l,t)
plt.size(drill_plot_enums.obs_topside_err_tild_q, :) =    [(1/4)*plt.screen_width, (1/5)*plt.screen_height];% Observer topside estimate errors \tilde{q}(l,t)
plt.size(drill_plot_enums.bilat_obs_gain_P_pp, :) =       [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% Bilateral observer gain P^{++}(x)
plt.size(drill_plot_enums.bilat_obs_gain_P_pm, :) =       [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% Bilateral observer gain P^{+-}(x)
plt.size(drill_plot_enums.bilat_obs_gain_P_mp, :) =       [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% Bilateral observer gain P^{-+}(x)
plt.size(drill_plot_enums.bilat_obs_gain_P_mm, :) =       [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% Bilateral observer gain P^{--}(x)
plt.size(drill_plot_enums.unilat_obs_gain_P_p, :) =       [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% Unilateral observer gain P^+(x)
plt.size(drill_plot_enums.unilat_obs_gain_P_m, :) =       [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% Unilateral observer gain P^-(x)
plt.size(drill_plot_enums.kernel_M11, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% M11 kernel equation solution
plt.size(drill_plot_enums.kernel_M12, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% M12 kernel equation solution
plt.size(drill_plot_enums.kernel_M21, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% M21 kernel equation solution
plt.size(drill_plot_enums.kernel_M22, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% M22 kernel equation solution
plt.size(drill_plot_enums.kernel_N11, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% N11 kernel equation solution
plt.size(drill_plot_enums.kernel_N12, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% N12 kernel equation solution
plt.size(drill_plot_enums.kernel_N21, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% N21 kernel equation solution
plt.size(drill_plot_enums.kernel_N22, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% N22 kernel equation solution
plt.size(drill_plot_enums.kernel_K21, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% K21 kernel equation solution
plt.size(drill_plot_enums.kernel_M, :)   =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% M kernel equation solution
plt.size(drill_plot_enums.kernel_N, :)   =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% N kernel equation solution
%% 2X2 system parameters

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2X2 system parameters(ttt = two times two)
% 
% x \in [0,1], t \in [0,\infty)
%
% u_t(x,t) + \lambda u_x(x,t) = \sigma_+(x)v(x,t)
% v_t(x,t) - \mu     v_x(x,t) = \sigma_-(x)u(x,t)
%
% with boundary conditions
%
% u(0,t) = Q v(0,t) + U1(t)
% v(1,t) = R u(1,t) + U2(t)
%
% and initial conditions
%
% u(x,0) = u0(x);
% v(x,0) = v0(x);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ttt.lambda = (1/drill.l)*sqrt(drill.beta/drill.rho);
ttt.mu = (1/drill.l)*sqrt(drill.beta/drill.rho);
ttt.sigma_p_handle = @(x)((-1/2)*(drill.F_1/drill.rho)*exp(((drill.l*drill.F_1)/sqrt(drill.beta*drill.rho))*x)); % Definition of sigma_+(x)
ttt.sigma_m_handle = @(x)((-1/2)*(drill.F_1/drill.rho)*exp(-((drill.l*drill.F_1)/sqrt(drill.beta*drill.rho))*x));% Definition of sigma_-(x)
ttt.sigma_p(1:sys.N_g_sim) = ttt.sigma_p_handle(sys.xspan_sim);
ttt.sigma_m(1:sys.N_g_sim) = ttt.sigma_m_handle(sys.xspan_sim);
ttt.Q = -(1-(drill.J*(sqrt(drill.beta*drill.rho)/drill.A_1)))/(1+(drill.J*(sqrt(drill.beta*drill.rho))/drill.A_1));
ttt.R = exp(-(drill.l*drill.F_1)/sqrt(drill.beta*drill.rho));%R same as \rho in report
ttt.U1 = @(t)((1/(1+drill.J*(sqrt(drill.beta*drill.rho)/drill.A_1)))*(drill.q_bit(t) + drill.J*drill.p_sp));
ttt.Theta = @(t)((drill.J/(1+drill.J*(sqrt(drill.beta*drill.rho)/drill.A_1)))*drill.p_r(t));%External disturbance entering at left boundary
ttt.U2 = @(t)((drill.A_1/(sqrt(drill.beta*drill.rho)))*(drill.p_l(t) - drill.p_sp + drill.rho*drill.g*drill.l)*exp(-(drill.l*drill.F_1)/sqrt(drill.beta*drill.rho)));
ttt.u0_handle = @(x)((1/2)*(drill.q0(drill.l*x) + (drill.A_1/sqrt(drill.beta*drill.rho))*(drill.p0(drill.l*x) - drill.p_sp + drill.rho*drill.g*drill.l*x)).*exp(((drill.l*drill.F_1)/(2*sqrt(drill.beta*drill.rho)))*x));
ttt.v0_handle = @(x)((1/2)*(drill.q0(drill.l*x) - (drill.A_1/sqrt(drill.beta*drill.rho))*(drill.p0(drill.l*x) - drill.p_sp + drill.rho*drill.g*drill.l*x)).*exp(-((drill.l*drill.F_1)/(2*sqrt(drill.beta*drill.rho)))*x));
ttt.u0(1:sys.N_g_sim) = ttt.u0_handle(sys.xspan_sim);
ttt.v0(1:sys.N_g_sim) = ttt.v0_handle(sys.xspan_sim);

ttt.distDelay = drill.distDelay;%Time delay for observers to "estimate" disturbance
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sys.drill = drill;
sys.ttt = ttt;

clear drill;
clear ttt;

%% Observer gain calculations
% Bilateral observer gains(obsb => bilateral observer)
[obsb.P_pp, obsb.P_pm, obsb.P_mp, obsb.P_mm, obsb.M11, obsb.M12, obsb.M21, obsb.M22, obsb.N11, obsb.N12, obsb.N21, obsb.N22, obsb.K21] = bilateral_2t2_obs_gain_case_1(sys);%Case I or IV since \lambda = \mu, x_s = 0.5
obsb.conv_time = max([(1/(sys.ttt.mu)) (1/(sys.ttt.lambda))]);

% Right sided observer gains(obss => single-sided observer, y = u(1,t))
[obss.P_p, obss.P_m, obss.M, obss.N] = right_2t2_obs_gain(sys);
obss.conv_time = (1/(sys.ttt.mu)) + (1/(sys.ttt.lambda));

%% Simulation loop

% Initial conditions for bilateral observer
obsb.un = sys.ttt.u0;
obsb.vn = sys.ttt.v0;

% Initial conditions for right boundary observer
obss.un = sys.ttt.u0;
obss.vn = sys.ttt.v0;

[sys, obss, obsb] = sim_loop_u_v(sys, obss, obsb);
%% Plots preprocessing
% Reduce number of elements in M and N
M(1:sys.N_g, 1:sys.N_g) = obss.M(1:2:sys.N_g_sim, 1:2:sys.N_g_sim);
N(1:sys.N_g, 1:sys.N_g) = obss.N(1:2:sys.N_g_sim, 1:2:sys.N_g_sim);

%Remove all kernel terms outside triangular domains
 for j=1:sys.N_g
        for i=j+1:sys.N_g
            obsb.M11(j,i) = nan;
            obsb.M12(j,i) = nan;
            obsb.M21(j,i) = nan;
            obsb.M22(j,i) = nan;
            obsb.N11(j,i) = nan;
            obsb.N12(j,i) = nan;
            obsb.N21(j,i) = nan;
            obsb.N22(j,i) = nan;
            M(j,i) = nan;
            N(j,i) = nan;
        end
 end
 
 % Calculate drilling values
sys.drill.q = zeros(sys.N_sim, sys.N_g_sim);
sys.drill.p = zeros(sys.N_sim, sys.N_g_sim);
obsb.q = zeros(sys.N_sim, sys.N_g_sim);
obsb.p = zeros(sys.N_sim, sys.N_g_sim);
obss.q = zeros(sys.N_sim, sys.N_g_sim);
obsb.p = zeros(sys.N_sim, sys.N_g_sim);
obsb.q_error = zeros(sys.N_sim, sys.N_g_sim);
obsb.p_error = zeros(sys.N_sim, sys.N_g_sim);
obss.q_error = zeros(sys.N_sim, sys.N_g_sim);
obss.p_error = zeros(sys.N_sim, sys.N_g_sim);
obsb.q_error_norm = zeros(1, sys.N_sim);
obsb.p_error_norm = zeros(1, sys.N_sim);
obss.q_error_norm = zeros(1, sys.N_sim);
obss.p_error_norm = zeros(1, sys.N_sim);
 

sys.drill.q = (sys.ttt.u).*exp(-((sys.drill.l*sys.drill.F_1)/(2*sqrt(sys.drill.beta*sys.drill.rho)))*sys.xspan_sim) + (sys.ttt.v).*exp(((sys.drill.l*sys.drill.F_1)/(2*sqrt(sys.drill.beta*sys.drill.rho)))*sys.xspan_sim);
sys.drill.p = ones(sys.N_sim, sys.N_g_sim)*sys.drill.p_sp - sys.drill.rho*sys.drill.g*kron(ones(sys.N_sim, 1), sys.zspan_sim) + (sqrt(sys.drill.beta*sys.drill.rho)/sys.drill.A_1)*((sys.ttt.u).*exp(-((sys.drill.l*sys.drill.F_1)/(2*sqrt(sys.drill.beta*sys.drill.rho)))*sys.xspan_sim) - (sys.ttt.v).*exp(((sys.drill.l*sys.drill.F_1)/(2*sqrt(sys.drill.beta*sys.drill.rho)))*sys.xspan_sim)); 

obsb.q = (obsb.u).*exp(-((sys.drill.l*sys.drill.F_1)/(2*sqrt(sys.drill.beta*sys.drill.rho)))*sys.xspan_sim) + (obsb.v).*exp(((sys.drill.l*sys.drill.F_1)/(2*sqrt(sys.drill.beta*sys.drill.rho)))*sys.xspan_sim);
obsb.p = ones(sys.N_sim, sys.N_g_sim)*sys.drill.p_sp - sys.drill.rho*sys.drill.g*kron(ones(sys.N_sim, 1), sys.zspan_sim) + (sqrt(sys.drill.beta*sys.drill.rho)/sys.drill.A_1)*((obsb.u).*exp(-((sys.drill.l*sys.drill.F_1)/(2*sqrt(sys.drill.beta*sys.drill.rho)))*sys.xspan_sim) - (obsb.v).*exp(((sys.drill.l*sys.drill.F_1)/(2*sqrt(sys.drill.beta*sys.drill.rho)))*sys.xspan_sim));

obss.q = (obss.u).*exp(-((sys.drill.l*sys.drill.F_1)/(2*sqrt(sys.drill.beta*sys.drill.rho)))*sys.xspan_sim) + (obss.v).*exp(((sys.drill.l*sys.drill.F_1)/(2*sqrt(sys.drill.beta*sys.drill.rho)))*sys.xspan_sim);
obss.p = ones(sys.N_sim, sys.N_g_sim)*sys.drill.p_sp - sys.drill.rho*sys.drill.g*kron(ones(sys.N_sim, 1), sys.zspan_sim) + (sqrt(sys.drill.beta*sys.drill.rho)/sys.drill.A_1)*((obss.u).*exp(-((sys.drill.l*sys.drill.F_1)/(2*sqrt(sys.drill.beta*sys.drill.rho)))*sys.xspan_sim) - (obss.v).*exp(((sys.drill.l*sys.drill.F_1)/(2*sqrt(sys.drill.beta*sys.drill.rho)))*sys.xspan_sim));

obsb.q_error = sys.drill.q - obsb.q;
obsb.p_error = sys.drill.p - obsb.p;
obss.q_error = sys.drill.q - obss.q;
obss.p_error = sys.drill.p - obss.p;

obsb.q_error_squared = obsb.q_error.^2;
obsb.p_error_squared = obsb.p_error.^2;
obss.q_error_squared = obss.q_error.^2;
obss.p_error_squared = obss.p_error.^2;

obsb.q_error_norm = sqrt(trapz(sys.zspan_sim, obsb.q_error_squared, 2)/(sys.drill.l));
obsb.p_error_norm = sqrt(trapz(sys.zspan_sim, obsb.p_error_squared, 2)/(sys.drill.l));
obss.q_error_norm = sqrt(trapz(sys.zspan_sim, obss.q_error_squared, 2)/(sys.drill.l));
obss.p_error_norm = sqrt(trapz(sys.zspan_sim, obss.p_error_squared, 2)/(sys.drill.l));
%% Plots
if(plt.flags(drill_plot_enums.drill_state_p))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.drill_state_p, 1), plt.positions(drill_plot_enums.drill_state_p, 2), plt.size(drill_plot_enums.drill_state_p, 1), plt.size(drill_plot_enums.drill_state_p,2)]); 
    s = surf(sys.zspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), sys.drill.p(sys.t_start_grid:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Pressure $p(z,t)$ of mud in drill string');
    xlabel('Position $z$ in drill string');
    ylabel('Time $t$');
    zlabel('Pressure $p$ [Pa]');
    axis(plt.axis_set_p_surf);
    view(plt.surf_view_time);
end

if(plt.flags(drill_plot_enums.drill_state_q))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.drill_state_q, 1), plt.positions(drill_plot_enums.drill_state_q, 2), plt.size(drill_plot_enums.drill_state_q, 1), plt.size(drill_plot_enums.drill_state_q,2)]);
    s = surf(sys.zspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), sys.drill.q(sys.t_start_grid:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Volumetric flow $q(z,t)$ of mud in drill string');
    xlabel('Position $z$ in drill string');
    ylabel('Time $t$');
    zlabel('Volumetric flow $q$ [$m^3/s$]');
    axis(plt.axis_set_q_surf);
    view(plt.surf_view_time);
end

if(plt.flags(drill_plot_enums.bilat_obs_est_hat_p))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.bilat_obs_est_hat_p, 1), plt.positions(drill_plot_enums.bilat_obs_est_hat_p, 2), plt.size(drill_plot_enums.bilat_obs_est_hat_p, 1), plt.size(drill_plot_enums.bilat_obs_est_hat_p,2)]); 
    s = surf(sys.zspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.p(sys.t_start_grid:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('WDP observer pressure estimate $\hat{p}(z,t)$ of mud in drill string');
    xlabel('Position $z$ in drill string');
    ylabel('Time $t$');
    zlabel('Pressure estimate $\hat{p}$ [Pa]');
    axis(plt.axis_set_p_surf);
    view(plt.surf_view_time);
end

if(plt.flags(drill_plot_enums.bilat_obs_est_hat_q))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.bilat_obs_est_hat_q, 1), plt.positions(drill_plot_enums.bilat_obs_est_hat_q, 2), plt.size(drill_plot_enums.bilat_obs_est_hat_q, 1), plt.size(drill_plot_enums.bilat_obs_est_hat_q,2)]);
    s = surf(sys.zspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.q(sys.t_start_grid:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('WDP observer volumetric flow estimate $\hat{q}(z,t)$ of mud in drill string');
    xlabel('Position $z$ in drill string');
    ylabel('Time $t$');
    zlabel('Volumetric flow estimate $\hat{q}$ [$m^3/s$]');
    axis(plt.axis_set_q_surf);
    view(plt.surf_view_time);
end

if(plt.flags(drill_plot_enums.unilat_obs_est_hat_p))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.unilat_obs_est_hat_p, 1), plt.positions(drill_plot_enums.unilat_obs_est_hat_p, 2), plt.size(drill_plot_enums.unilat_obs_est_hat_p, 1), plt.size(drill_plot_enums.unilat_obs_est_hat_p,2)]); 
    s = surf(sys.zspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.p(sys.t_start_grid:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Topside observer pressure estimate $\hat{p}(z,t)$ of mud in drill string');
    xlabel('Position $z$ in drill string');
    ylabel('Time $t$');
    zlabel('Pressure estimate $\hat{p}$ [Pa]');
    axis(plt.axis_set_p_surf);
    view(plt.surf_view_time);
end

if(plt.flags(drill_plot_enums.unilat_obs_est_hat_q))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.unilat_obs_est_hat_q, 1), plt.positions(drill_plot_enums.unilat_obs_est_hat_q, 2), plt.size(drill_plot_enums.unilat_obs_est_hat_q, 1), plt.size(drill_plot_enums.unilat_obs_est_hat_q,2)]);
    s = surf(sys.zspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.q(sys.t_start_grid:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Topside observer volumetric flow estimate $\hat{q}(z,t)$ of mud in drill string');
    xlabel('Position $z$ in drill string');
    ylabel('Time $t$');
    zlabel('Volumetric flow estimate $\hat{q}$ [$m^3/s$]');
    axis(plt.axis_set_q_surf);
    view(plt.surf_view_time);
end          

if(plt.flags(drill_plot_enums.bilat_obs_est_err_tild_p))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.bilat_obs_est_err_tild_p, 1), plt.positions(drill_plot_enums.bilat_obs_est_err_tild_p, 2), plt.size(drill_plot_enums.bilat_obs_est_err_tild_p, 1), plt.size(drill_plot_enums.bilat_obs_est_err_tild_p,2)]); 
    s = surf(sys.zspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.p_error(sys.t_start_grid:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('WDP observer pressure estimate error $\tilde{p}(z,t)$ of mud in drill string');
    xlabel('Position $z$ in drill string');
    ylabel('Time $t$');
    zlabel('Pressure estimate error $\tilde{p}$ [Pa]');
    axis(plt.axis_set_p_err_surf);
    view(plt.surf_view_time);
end

if(plt.flags(drill_plot_enums.bilat_obs_est_err_tild_q))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.bilat_obs_est_err_tild_q, 1), plt.positions(drill_plot_enums.bilat_obs_est_err_tild_q, 2), plt.size(drill_plot_enums.bilat_obs_est_err_tild_q, 1), plt.size(drill_plot_enums.bilat_obs_est_err_tild_q,2)]);
    s = surf(sys.zspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.q_error(sys.t_start_grid:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('WDP observer volumetric flow estimate error $\tilde{q}(z,t)$ of mud in drill string');
    xlabel('Position $z$ in drill string');
    ylabel('Time $t$');
    zlabel('Volumetric flow estimate error $\tilde{q}$ [$m^3/s$]');
    axis(plt.axis_set_q_err_surf);
    view(plt.surf_view_time);
end

if(plt.flags(drill_plot_enums.unilat_obs_est_err_tild_p))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.unilat_obs_est_err_tild_p, 1), plt.positions(drill_plot_enums.unilat_obs_est_err_tild_p, 2), plt.size(drill_plot_enums.unilat_obs_est_err_tild_p, 1), plt.size(drill_plot_enums.unilat_obs_est_err_tild_p,2)]); 
    s = surf(sys.zspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.p_error(sys.t_start_grid:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Topside observer pressure estimate error $\tilde{p}(z,t)$ of mud in drill string');
    xlabel('Position $z$ in drill string');
    ylabel('Time $t$');
    zlabel('Pressure estimate error $\tilde{p}$ [Pa]');
    axis(plt.axis_set_p_err_surf);
    view(plt.surf_view_time);
end

if(plt.flags(drill_plot_enums.unilat_obs_est_err_tild_q))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.unilat_obs_est_err_tild_q, 1), plt.positions(drill_plot_enums.unilat_obs_est_err_tild_q, 2), plt.size(drill_plot_enums.unilat_obs_est_err_tild_q, 1), plt.size(drill_plot_enums.unilat_obs_est_err_tild_q,2)]);
    s = surf(sys.zspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.q_error(sys.t_start_grid:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Topside observer volumetric flow estimate error $\tilde{q}(z,t)$ of mud in drill string');
    xlabel('Position $z$ in drill string');
    ylabel('Time $t$');
    zlabel('Volumetric flow estimate error $\tilde{q}$ [$m^3/s$]');
    axis(plt.axis_set_q_err_surf);
    view(plt.surf_view_time);
end

if(plt.flags(drill_plot_enums.obs_err_norm_p))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.obs_err_norm_p, 1), plt.positions(drill_plot_enums.obs_err_norm_p, 2), plt.size(drill_plot_enums.obs_err_norm_p, 1), plt.size(drill_plot_enums.obs_err_norm_p,2)]);
    hold on;
    p_bilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.p_error_norm(sys.t_start_grid:sys.t_skip:sys.N_sim));
    p_bilateral.Color = 'blue';
    p_bilateral.LineWidth = 2;
    p_unilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.p_error_norm(sys.t_start_grid:sys.t_skip:sys.N_sim));
    p_unilateral.Color = 'red';
    p_unilateral.LineWidth = 2;
    line([sys.drill.kick_time sys.drill.kick_time], [0 20*10^6], 'Color', 'black', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay) (sys.drill.kick_time + sys.drill.distDelay)], [0 20*10^6], 'Color', 'magenta', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time)], [0 20*10^6], 'Color', 'blue', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obss.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obss.conv_time)], [0 20*10^6], 'Color', 'red', 'LineStyle', '--');
    title('Topside and WDP observer $||\tilde{p}(t)||$');
    xlabel('Time $t$');
    ylabel('Norm of pressure estimation error $||\tilde{p}||$ [Pa]');
    legend('WDP observer $||\tilde{p}(t)||$', 'Topside observer $||\tilde{p}(t)||$','Time when kick occurs', 'Time when correct reservoir pressure estimate $\hat{p}_r$ available', 'WDP observer $t_F = \frac{1}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available', 'Topside observer $t_F = \frac{2}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available');
    axis(plt.axis_set_p_err_plot);
end

if(plt.flags(drill_plot_enums.obs_err_norm_q))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.obs_err_norm_q, 1), plt.positions(drill_plot_enums.obs_err_norm_q, 2), plt.size(drill_plot_enums.obs_err_norm_q, 1), plt.size(drill_plot_enums.obs_err_norm_q,2)]);
    hold on;
    p_bilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.q_error_norm(sys.t_start_grid:sys.t_skip:sys.N_sim));
    p_bilateral.Color = 'blue';
    p_bilateral.LineWidth = 2;
    p_unilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.q_error_norm(sys.t_start_grid:sys.t_skip:sys.N_sim));
    p_unilateral.Color = 'red';
    p_unilateral.LineWidth = 2;
    line([sys.drill.kick_time sys.drill.kick_time], [-0.1 0.4], 'Color', 'black', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay) (sys.drill.kick_time + sys.drill.distDelay)], [-0.1 0.4], 'Color', 'magenta', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time)], [-0.1 0.4], 'Color', 'blue', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obss.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obss.conv_time)], [-0.1 0.4], 'Color', 'red', 'LineStyle', '--');
    title('Topside and WDP observer $||\tilde{q}(t)||$');
    xlabel('Time $t$');
    ylabel('Norm of volumetric flow estimation error $||\tilde{q}||$ [$m^3/s$]');
    legend('WDP observer $||\tilde{q}(t)||$', 'Topside observer $||\tilde{q}(t)||$','Time when kick occurs', 'Time when correct reservoir pressure estimate $\hat{p}_r$ available', 'WDP observer $t_F = \frac{1}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available', 'Topside observer $t_F = \frac{2}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available');
    axis(plt.axis_set_q_err_plot);
end

if(plt.flags(drill_plot_enums.obs_downhole_est_p))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.obs_downhole_est_p, 1), plt.positions(drill_plot_enums.obs_downhole_est_p, 2), plt.size(drill_plot_enums.obs_downhole_est_p, 1), plt.size(drill_plot_enums.obs_downhole_est_p,2)]);
    hold on;
    p_actual = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), sys.drill.p(sys.t_start_grid:sys.t_skip:sys.N_sim, 1));
    p_actual.Color = 'black';
    p_actual.LineWidth = 2;
    p_bilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.p(sys.t_start_grid:sys.t_skip:sys.N_sim, 1));
    p_bilateral.Color = 'blue';
    p_bilateral.LineWidth = 2;
    p_unilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.p(sys.t_start_grid:sys.t_skip:sys.N_sim, 1));
    p_unilateral.Color = 'red';
    p_unilateral.LineWidth = 2;
    line([sys.drill.kick_time sys.drill.kick_time], [-10^7 11*10^7], 'Color', 'black', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay) (sys.drill.kick_time + sys.drill.distDelay)], [-10^7 11*10^7], 'Color', 'magenta', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time)], [-10^7 11*10^7], 'Color', 'blue', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obss.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obss.conv_time)], [-10^7 11*10^7], 'Color', 'red', 'LineStyle', '--');
    title('Topside and WDP observer $\hat{p}(0,t)$');
    xlabel('Time $t$');
    ylabel('Downhole pressure estimate $\hat{p}(0,t)$ [Pa]');
    legend('Downhole pressure $p(0,t)$', 'WDP observer $\hat{p}(0,t)$', 'Topside observer $\hat{p}(0,t)$','Time when kick occurs', 'Time when correct reservoir pressure estimate $\hat{p}_r$ available', 'WDP observer $t_F = \frac{1}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available', 'Topside observer $t_F = \frac{2}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available', 'Location', 'southeast');
    axis(plt.axis_set_p_plot);
    %axis([19 35 6*10^7 9*10^7]);
end

if(plt.flags(drill_plot_enums.obs_downhole_est_q))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.obs_downhole_est_q, 1), plt.positions(drill_plot_enums.obs_downhole_est_q, 2), plt.size(drill_plot_enums.obs_downhole_est_q, 1), plt.size(drill_plot_enums.obs_downhole_est_q,2)]);
    hold on;
    p_actual = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), sys.drill.q(sys.t_start_grid:sys.t_skip:sys.N_sim, 1));
    p_actual.Color = 'black';
    p_actual.LineWidth = 2;
    p_bilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.q(sys.t_start_grid:sys.t_skip:sys.N_sim, 1));
    p_bilateral.Color = 'blue';
    p_bilateral.LineWidth = 2;
    p_unilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.q(sys.t_start_grid:sys.t_skip:sys.N_sim, 1));
    p_unilateral.Color = 'red';
    p_unilateral.LineWidth = 2;
    line([sys.drill.kick_time sys.drill.kick_time], [0 1.5], 'Color', 'black', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay) (sys.drill.kick_time + sys.drill.distDelay)], [0 1.5], 'Color', 'magenta', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time)], [0 1.5], 'Color', 'blue', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obss.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obss.conv_time)], [0 1.5], 'Color', 'red', 'LineStyle', '--');
    title('Topside and WDP observer $\hat{q}(0,t)$');
    xlabel('Time $t$');
    ylabel('Downhole volumetric flow estimate $\hat{q}(0,t)$ [Pa]');
    legend('Downhole volumetric flow $q(0,t)$', 'WDP observer $\hat{q}(0,t)$', 'Topside observer $\hat{q}(0,t)$','Time when kick occurs', 'Time when correct reservoir pressure estimate $\hat{p}_r$ available', 'WDP observer $t_F = \frac{1}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available', 'Topside observer $t_F = \frac{2}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available', 'Location', 'southeast');
    axis(plt.axis_set_q_plot);
end

if(plt.flags(drill_plot_enums.obs_downhole_err_tild_p))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.obs_downhole_err_tild_p, 1), plt.positions(drill_plot_enums.obs_downhole_err_tild_p, 2), plt.size(drill_plot_enums.obs_downhole_err_tild_p, 1), plt.size(drill_plot_enums.obs_downhole_err_tild_p,2)]);
    hold on;
    p_bilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.p_error(sys.t_start_grid:sys.t_skip:sys.N_sim, 1));
    p_bilateral.Color = 'blue';
    p_bilateral.LineWidth = 2;
    p_unilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.p_error(sys.t_start_grid:sys.t_skip:sys.N_sim, 1));
    p_unilateral.Color = 'red';
    p_unilateral.LineWidth = 2;
    line([sys.drill.kick_time sys.drill.kick_time], [0 25*10^6], 'Color', 'black', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay) (sys.drill.kick_time + sys.drill.distDelay)], [0 25*10^6], 'Color', 'magenta', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time)], [0 25*10^6], 'Color', 'blue', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obss.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obss.conv_time)], [0 25*10^6], 'Color', 'red', 'LineStyle', '--');
    title('Topside and WDP observer $\tilde{p}(0,t)$');
    xlabel('Time $t$');
    ylabel('Downhole pressure estimate error $\tilde{p}(0,t)$ [Pa]');
    legend('WDP observer $\tilde{p}(0,t)$', 'Topside observer $\tilde{p}(0,t)$','Time when kick occurs', 'Time when correct reservoir pressure estimate $\hat{p}_r$ available', 'WDP observer $t_F = \frac{1}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available', 'Topside observer $t_F = \frac{2}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available');
    axis(plt.axis_set_p_err_plot);
end

if(plt.flags(drill_plot_enums.obs_downhole_err_tild_q))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.obs_downhole_err_tild_q, 1), plt.positions(drill_plot_enums.obs_downhole_err_tild_q, 2), plt.size(drill_plot_enums.obs_downhole_err_tild_q, 1), plt.size(drill_plot_enums.obs_downhole_err_tild_q,2)]);
    hold on;
    p_bilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.q_error(sys.t_start_grid:sys.t_skip:sys.N_sim, 1));
    p_bilateral.Color = 'blue';
    p_bilateral.LineWidth = 2;
    p_unilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.q_error(sys.t_start_grid:sys.t_skip:sys.N_sim, 1));
    p_unilateral.Color = 'red';
    p_unilateral.LineWidth = 2;
    line([sys.drill.kick_time sys.drill.kick_time], [-0.15 0.4], 'Color', 'black', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay) (sys.drill.kick_time + sys.drill.distDelay)], [-0.15 0.4], 'Color', 'magenta', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time)], [-0.15 0.4], 'Color', 'blue', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obss.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obss.conv_time)], [-0.15 0.4], 'Color', 'red', 'LineStyle', '--');
    title('Topside and WDP observer $\tilde{q}(0,t)$');
    xlabel('Time $t$');
    ylabel('Downhole volumetric flow estimate error $\tilde{q}(0,t)$ [$m^3/s$]');
    legend('WDP observer $\tilde{q}(0,t)$', 'Topside observer $\tilde{q}(0,t)$','Time when kick occurs', 'Time when correct reservoir pressure estimate $\hat{p}_r$ available', 'WDP observer $t_F = \frac{1}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available', 'Topside observer $t_F = \frac{2}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available');
    axis(plt.axis_set_q_err_plot);
end

if(plt.flags(drill_plot_enums.obs_topside_est_p))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.obs_topside_est_p, 1), plt.positions(drill_plot_enums.obs_topside_est_p, 2), plt.size(drill_plot_enums.obs_topside_est_p, 1), plt.size(drill_plot_enums.obs_topside_est_p,2)]);
    hold on;
    p_actual = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), sys.drill.p(sys.t_start_grid:sys.t_skip:sys.N_sim, sys.N_g_sim));
    p_actual.Color = 'black';
    p_actual.LineWidth = 2;
    p_bilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.p(sys.t_start_grid:sys.t_skip:sys.N_sim, sys.N_g_sim));
    p_bilateral.Color = 'blue';
    p_bilateral.LineWidth = 2;
    p_unilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.p(sys.t_start_grid:sys.t_skip:sys.N_sim, sys.N_g_sim));
    p_unilateral.Color = 'red';
    p_unilateral.LineWidth = 2;
    line([sys.drill.kick_time sys.drill.kick_time], [-10^7 11*10^7], 'Color', 'black', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay) (sys.drill.kick_time + sys.drill.distDelay)], [-10^7 11*10^7], 'Color', 'magenta', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time)], [-10^7 11*10^7], 'Color', 'blue', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obss.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obss.conv_time)], [-10^7 11*10^7], 'Color', 'red', 'LineStyle', '--');
    title('Topside and WDP observer $\hat{p}(l,t)$');
    xlabel('Time $t$');
    ylabel('Topside pressure estimate $\hat{p}(l,t)$ [Pa]');
    legend('Topside pressure $p(l,t)$', 'WDP observer $\hat{p}(l,t)$', 'Topside observer $\hat{p}(l,t)$','Time when kick occurs', 'Time when correct reservoir pressure estimate $\hat{p}_r$ available', 'WDP observer $t_F = \frac{1}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available', 'Topside observer $t_F = \frac{2}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available');
    axis(plt.axis_set_p_plot);
end

if(plt.flags(drill_plot_enums.obs_topside_est_q))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.obs_topside_est_q, 1), plt.positions(drill_plot_enums.obs_topside_est_q, 2), plt.size(drill_plot_enums.obs_topside_est_q, 1), plt.size(drill_plot_enums.obs_topside_est_q,2)]);
    hold on;
    p_actual = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), sys.drill.q(sys.t_start_grid:sys.t_skip:sys.N_sim, sys.N_g_sim));
    p_actual.Color = 'black';
    p_actual.LineWidth = 2;
    p_bilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.q(sys.t_start_grid:sys.t_skip:sys.N_sim, sys.N_g_sim));
    p_bilateral.Color = 'blue';
    p_bilateral.LineWidth = 2;
    p_unilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.q(sys.t_start_grid:sys.t_skip:sys.N_sim, sys.N_g_sim));
    p_unilateral.Color = 'red';
    p_unilateral.LineWidth = 2;
    line([sys.drill.kick_time sys.drill.kick_time], [0 1.5], 'Color', 'black', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay) (sys.drill.kick_time + sys.drill.distDelay)], [0 1.5], 'Color', 'magenta', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time)], [0 1.5], 'Color', 'blue', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obss.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obss.conv_time)], [0 1.5], 'Color', 'red', 'LineStyle', '--');
    title('Topside and WDP observer $\hat{q}(l,t)$');
    xlabel('Time $t$');
    ylabel('Topside volumetric flow estimate $\hat{q}(l,t)$ [Pa]');
    legend('Topside volumetric flow $q(l,t)$', 'WDP observer $\hat{q}(l,t)$', 'Topside observer $\hat{q}(l,t)$','Time when kick occurs', 'Time when correct reservoir pressure estimate $\hat{p}_r$ available', 'WDP observer $t_F = \frac{1}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available', 'Topside observer $t_F = \frac{2}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available', 'Location', 'southeast');
    axis(plt.axis_set_q_plot);
end

if(plt.flags(drill_plot_enums.obs_topside_err_tild_p))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.obs_topside_err_tild_p, 1), plt.positions(drill_plot_enums.obs_topside_err_tild_p, 2), plt.size(drill_plot_enums.obs_topside_err_tild_p, 1), plt.size(drill_plot_enums.obs_topside_err_tild_p,2)]);
    hold on;
    p_bilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.p_error(sys.t_start_grid:sys.t_skip:sys.N_sim, sys.N_g_sim));
    p_bilateral.Color = 'blue';
    p_bilateral.LineWidth = 2;
    p_unilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.p_error(sys.t_start_grid:sys.t_skip:sys.N_sim, sys.N_g_sim));
    p_unilateral.Color = 'red';
    p_unilateral.LineWidth = 2;
    line([sys.drill.kick_time sys.drill.kick_time], [0 25*10^6], 'Color', 'black', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay) (sys.drill.kick_time + sys.drill.distDelay)], [0 25*10^6], 'Color', 'magenta', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time)], [0 25*10^6], 'Color', 'blue', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obss.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obss.conv_time)], [0 25*10^6], 'Color', 'red', 'LineStyle', '--');
    title('Topside and WDP observer $\tilde{p}(l,t)$');
    xlabel('Time $t$');
    ylabel('Topside pressure estimate error $\tilde{p}(l,t)$ [Pa]');
    legend('WDP observer $\tilde{p}(l,t)$', 'Topside observer $\tilde{p}(l,t)$','Time when kick occurs', 'Time when correct reservoir pressure estimate $\hat{p}_r$ available', 'WDP observer $t_F = \frac{1}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available', 'Topside observer $t_F = \frac{2}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available');
    axis(plt.axis_set_p_err_plot);
end

if(plt.flags(drill_plot_enums.obs_topside_err_tild_q))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.obs_topside_err_tild_q, 1), plt.positions(drill_plot_enums.obs_topside_err_tild_q, 2), plt.size(drill_plot_enums.obs_topside_err_tild_q, 1), plt.size(drill_plot_enums.obs_topside_err_tild_q,2)]);
    hold on;
    p_bilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obsb.q_error(sys.t_start_grid:sys.t_skip:sys.N_sim, sys.N_g_sim));
    p_bilateral.Color = 'blue';
    p_bilateral.LineWidth = 2;
    p_unilateral = plot(sys.Tspan(sys.t_start_grid:sys.t_skip:sys.N_sim), obss.q_error(sys.t_start_grid:sys.t_skip:sys.N_sim, sys.N_g_sim));
    p_unilateral.Color = 'red';
    p_unilateral.LineWidth = 2;
    line([sys.drill.kick_time sys.drill.kick_time], [-0.15 0.4], 'Color', 'black', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay) (sys.drill.kick_time + sys.drill.distDelay)], [-0.15 0.4], 'Color', 'magenta', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obsb.conv_time)], [-0.15 0.4], 'Color', 'blue', 'LineStyle', '--');
    line([(sys.drill.kick_time + sys.drill.distDelay + obss.conv_time) (sys.drill.kick_time + sys.drill.distDelay + obss.conv_time)], [-0.15 0.4], 'Color', 'red', 'LineStyle', '--');
    title('Topside and WDP observer $\tilde{q}(l,t)$');
    xlabel('Time $t$');
    ylabel('Topside volumetric flow estimate error $\tilde{q}(l,t)$ [$m^3/s$]');
    legend('WDP observer $\tilde{q}(l,t)$', 'Topside observer $\tilde{q}(l,t)$','Time when kick occurs', 'Time when correct reservoir pressure estimate $\hat{p}_r$ available', 'WDP observer $t_F = \frac{1}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available', 'Topside observer $t_F = \frac{2}{l}\sqrt{\frac{\beta}{\rho}}$ after correct $\hat{p}_r$ available');
    axis(plt.axis_set_q_err_plot);
end


if(plt.flags(drill_plot_enums.bilat_obs_gain_P_pp))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.bilat_obs_gain_P_pp, 1), plt.positions(drill_plot_enums.bilat_obs_gain_P_pp, 2), plt.size(drill_plot_enums.bilat_obs_gain_P_pp, 1), plt.size(drill_plot_enums.bilat_obs_gain_P_pp,2)]);
    p = plot(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), obsb.P_pp(1:sys.x_skip:sys.N_g_sim));
    p.Color = [0 0.4 0.6];%'blue';
    p.LineWidth = 2;
    title('WDP observer gain $P^{++}(x)$');
    xlabel('Position $x$');
    ylabel('Observer gain $P^{++}$');
    axis(plt.axis_set_obs_gains);
end

if(plt.flags(drill_plot_enums.bilat_obs_gain_P_pm))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.bilat_obs_gain_P_pm, 1), plt.positions(drill_plot_enums.bilat_obs_gain_P_pm, 2), plt.size(drill_plot_enums.bilat_obs_gain_P_pm, 1), plt.size(drill_plot_enums.bilat_obs_gain_P_pm,2)]);
    p = plot(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), obsb.P_pm(1:sys.x_skip:sys.N_g_sim));
    p.Color = [0.6 0 0.4];%'red';
    p.LineWidth = 2;
    title('WDP observer gain $P^{+-}(x)$');
    xlabel('Position $x$');
    ylabel('Observer gain $P^{+-}$');
    axis(plt.axis_set_obs_gains);
end

if(plt.flags(drill_plot_enums.bilat_obs_gain_P_mp))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.bilat_obs_gain_P_mp, 1), plt.positions(drill_plot_enums.bilat_obs_gain_P_mp, 2), plt.size(drill_plot_enums.bilat_obs_gain_P_mp, 1), plt.size(drill_plot_enums.bilat_obs_gain_P_mp,2)]);
    p = plot(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), obsb.P_mp(1:sys.x_skip:sys.N_g_sim));
    p.Color = 'green';
    p.LineWidth = 2;
    title('WDP observer gain $P^{-+}(x)$');
    xlabel('Position $x$');
    ylabel('Observer gain $P^{-+}$');
    axis(plt.axis_set_obs_gains);
end

if(plt.flags(drill_plot_enums.bilat_obs_gain_P_mm))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.bilat_obs_gain_P_mm, 1), plt.positions(drill_plot_enums.bilat_obs_gain_P_mm, 2), plt.size(drill_plot_enums.bilat_obs_gain_P_mm, 1), plt.size(drill_plot_enums.bilat_obs_gain_P_mm,2)]);
    p = plot(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), obsb.P_mm(1:sys.x_skip:sys.N_g_sim));
    p.Color = 'yellow';
    p.LineWidth = 2;
    title('WDP observer gain $P^{--}(x)$');
    xlabel('Position $x$');
    ylabel('Observer gain $P^{--}$');
    axis(plt.axis_set_obs_gains);
end

if(plt.flags(drill_plot_enums.unilat_obs_gain_P_p))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.unilat_obs_gain_P_p, 1), plt.positions(drill_plot_enums.unilat_obs_gain_P_p, 2), plt.size(drill_plot_enums.unilat_obs_gain_P_p, 1), plt.size(drill_plot_enums.unilat_obs_gain_P_p,2)]);
    p = plot(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), obss.P_p(1:sys.x_skip:sys.N_g_sim));
    p.Color = 'magenta';
    p.LineWidth = 2;
    title('Topside observer gain $\bar{P}^+(x)$');
    xlabel('Position $x$');
    ylabel('Observer gain $\bar{P}^+$');
    axis(plt.axis_set_obs_gains);
end

if(plt.flags(drill_plot_enums.unilat_obs_gain_P_m))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.unilat_obs_gain_P_m, 1), plt.positions(drill_plot_enums.unilat_obs_gain_P_m, 2), plt.size(drill_plot_enums.unilat_obs_gain_P_m, 1), plt.size(drill_plot_enums.unilat_obs_gain_P_m,2)]);
    p = plot(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), obss.P_m(1:sys.x_skip:sys.N_g_sim));
    p.Color = 'cyan';
    p.LineWidth = 2;
    title('Topside observer gain $\bar{P}^-(x)$');
    xlabel('Position $x$');
    ylabel('Observer gain $\bar{P}^-$');
    axis(plt.axis_set_obs_gains);
end

if(plt.flags(drill_plot_enums.kernel_M11))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.kernel_M11, 1), plt.positions(drill_plot_enums.kernel_M11, 2), plt.size(drill_plot_enums.kernel_M11, 1), plt.size(drill_plot_enums.kernel_M11,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.M11(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('WDP observer kernel $M_{11}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $M_{11}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(drill_plot_enums.kernel_M12))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.kernel_M12, 1), plt.positions(drill_plot_enums.kernel_M12, 2), plt.size(drill_plot_enums.kernel_M12, 1), plt.size(drill_plot_enums.kernel_M12,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.M12(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('WDP observer kernel $M_{12}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $M_{12}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(drill_plot_enums.kernel_M21))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.kernel_M21, 1), plt.positions(drill_plot_enums.kernel_M21, 2), plt.size(drill_plot_enums.kernel_M21, 1), plt.size(drill_plot_enums.kernel_M21,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.M21(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('WDP observer kernel $M_{21}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $M_{21}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(drill_plot_enums.kernel_M22))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.kernel_M22, 1), plt.positions(drill_plot_enums.kernel_M22, 2), plt.size(drill_plot_enums.kernel_M22, 1), plt.size(drill_plot_enums.kernel_M22,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.M22(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('WDP observer kernel $M_{22}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $M_{22}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(drill_plot_enums.kernel_N11))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.kernel_N11, 1), plt.positions(drill_plot_enums.kernel_N11, 2), plt.size(drill_plot_enums.kernel_N11, 1), plt.size(drill_plot_enums.kernel_N11,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.N11(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('WDP observer kernel $N_{11}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $N_{11}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(drill_plot_enums.kernel_N12))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.kernel_N12, 1), plt.positions(drill_plot_enums.kernel_N12, 2), plt.size(drill_plot_enums.kernel_N12, 1), plt.size(drill_plot_enums.kernel_N12,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.N12(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('WDP observer kernel $N_{12}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $N_{12}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(drill_plot_enums.kernel_N21))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.kernel_N21, 1), plt.positions(drill_plot_enums.kernel_N21, 2), plt.size(drill_plot_enums.kernel_N21, 1), plt.size(drill_plot_enums.kernel_N21,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.N21(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('WDP observer kernel $N_{21}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $N_{21}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(drill_plot_enums.kernel_N22))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.kernel_N22, 1), plt.positions(drill_plot_enums.kernel_N22, 2), plt.size(drill_plot_enums.kernel_N22, 1), plt.size(drill_plot_enums.kernel_N22,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.N22(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('WDP observer kernel $N_{22}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $N_{22}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(drill_plot_enums.kernel_K21))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.kernel_K21, 1), plt.positions(drill_plot_enums.kernel_K21, 2), plt.size(drill_plot_enums.kernel_K21, 1), plt.size(drill_plot_enums.kernel_K21,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.K21(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('WDP observer kernel $K_{21}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $K_{21}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(drill_plot_enums.kernel_M))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.kernel_M, 1), plt.positions(drill_plot_enums.kernel_M, 2), plt.size(drill_plot_enums.kernel_M, 1), plt.size(drill_plot_enums.kernel_M,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), M(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('Topside observer kernel $\bar{M}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $\bar{M}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(drill_plot_enums.kernel_N))
    figure
    set(gcf,'units','pixels','position',[plt.positions(drill_plot_enums.kernel_N, 1), plt.positions(drill_plot_enums.kernel_N, 2), plt.size(drill_plot_enums.kernel_N, 1), plt.size(drill_plot_enums.kernel_N,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), N(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('Topside observer kernel $\bar{N}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $\bar{N}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end