function [P_pp, P_pm, P_mp, P_mm, M11, M12, M21, M22, N11, N12, N21, N22, K21] = bilateral_2t2_obs_gain_case_2(sys)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2+2 system parameters(tpt = two plus two)
%
% u1_t(x,t) + \lambda_1 u1_x(x,t) = \sigma_11^+-(x) v1(x,t) + \sigma_12^+-(x) v2(x,t)
% u2_t(x,t) + \lambda_2 u2_x(x,t) = \sigma_21^+-(x) v1(x,t) + \sigma_22^+1(x) v2(x,t)
% v1_t(x,t) - \mu_1     v1_x(x,t) = \sigma_11^-+(x) u1(x,t) + \sigma_12^-+(x) u2(x,t)
% v2_t(x,t) - \mu_2     v2_x(x,t) = \sigma_21^-+(x) u1(x,t) + \sigma_22^-+(x) u2(x,t)
%
% with boundary conditions
%
% u1(0,t) = Q11 v1(0,t) + Q12 v2(1,t) 
% u2(0,t) = Q21 v1(0,t) + Q22 v2(1,t)
% v1(1,t) = R11 u1(1,t) + R12 u2(1,t) + U1(t)
% v2(1,t) = R21 u1(1,t) + R22 u2(1,t) + U2(t)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% x_s = 0.5, Case II
tpt.lambda_1 = sys.ttt.mu/0.5;
tpt.lambda_2 = sys.ttt.lambda/0.5;
tpt.mu_1 = sys.ttt.lambda/0.5;
tpt.mu_2 = sys.ttt.mu/0.5;
tpt.sigma_11_pm = sys.ttt.sigma_m(sys.N_g:-1:1);
tpt.sigma_12_pm = zeros(1, sys.N_g);
tpt.sigma_21_pm = zeros(1, sys.N_g);
tpt.sigma_22_pm = sys.ttt.sigma_p(sys.N_g+1:sys.N_g_sim);
tpt.sigma_11_mp = sys.ttt.sigma_p(sys.N_g:-1:1);
tpt.sigma_12_mp = zeros(1, sys.N_g);
tpt.sigma_21_mp = zeros(1, sys.N_g);
tpt.sigma_22_mp = sys.ttt.sigma_m(sys.N_g+1:sys.N_g_sim);
tpt.Q11 = 0;
tpt.Q12 = 1;
tpt.Q21 = 1;
tpt.Q22 = 0;
tpt.R11 = sys.ttt.Q;
tpt.R12 = 0;
tpt.R21 = 0;
tpt.R22 = sys.ttt.R;
tpt.U1 = sys.ttt.U1;
tpt.U2 = sys.ttt.U2;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sys.tpt = tpt;
clear tpt;

[P11_plus, P12_plus, P21_plus, P22_plus, P11_minus, P12_minus, P21_minus, P22_minus, M11, M12, M21, M22, N11, N12, N21, N22, K21] = right_2p2_obs_gain_case_2_3(sys);

P_pp = zeros(1, sys.N_g_sim);
P_pm = zeros(1, sys.N_g_sim);
P_mp = zeros(1, sys.N_g_sim);
P_mm = zeros(1, sys.N_g_sim);

P_pp(1:sys.N_g) = P12_minus(sys.N_g:-1:1);
P_pp(sys.N_g+1:2*sys.N_g) = P22_plus;

P_pm(1:sys.N_g) = P11_minus(sys.N_g:-1:1);
P_pm(sys.N_g+1:2*sys.N_g) = P21_plus;

P_mp(1:sys.N_g) = P12_plus(sys.N_g:-1:1);
P_mp(sys.N_g+1:2*sys.N_g) = P22_minus;

P_mm(1:sys.N_g) = P11_plus(sys.N_g:-1:1);
P_mm(sys.N_g+1:2*sys.N_g) = P21_minus;

end