function [P11_plus, P12_plus, P21_plus, P22_plus, P11_minus, P12_minus, P21_minus, P22_minus, M11, M12, M21, M22, N11, N12, N21, N22, K21] = right_2p2_obs_gain_case_1_4(sys)
    %% Kernel solving
    % First set Volterra kernels
    [N11, N21, M11, M21] = first_solver_case_1_4(sys);
    N11 = flipud(fliplr(N11));%zeros(sys.N_g);
    N21 = N21';
    M11 = M11';
    M21 = flipud(fliplr(M21));%zeros(sys.N_g);

    % Second set Volterra kernels
    [N12, N22, M12, M22] = second_solver_case_1_4(sys);
    N12 = N12';
    N22 = N22';
    M12 = M12';
    M22 = M22';

    sys.H21 = N21(:,1)' - M21(:,1)';
    
    % Fredholm kernel
    K21 = third_solver(sys);


    %% 2+2 observer gain calculations

    T21 = sys.tpt.lambda_1*K21(sys.N_g,:);

    M12_int = zeros(1, sys.N_g);
    M22_int = zeros(1, sys.N_g);
    N12_int = zeros(1, sys.N_g);
    N22_int = zeros(1, sys.N_g);

    for i=1:sys.N_g
        M12_int(i) = trapz(sys.xspan, M12(:,i).*(T21'));
        M22_int(i) = trapz(sys.xspan, M22(:,i).*(T21'));
        N12_int(i) = trapz(sys.xspan, N12(:,i).*(T21'));
        N22_int(i) = trapz(sys.xspan, N22(:,i).*(T21'));
    end

    P11_plus = zeros(1, sys.N_g);
    P12_plus = zeros(1, sys.N_g);
    P21_plus = zeros(1, sys.N_g);
    P22_plus = zeros(1, sys.N_g);
    P11_minus = zeros(1, sys.N_g);
    P12_minus = zeros(1, sys.N_g);
    P21_minus = zeros(1, sys.N_g);
    P22_minus = zeros(1, sys.N_g);


    for i=1:sys.N_g
        P11_plus(i) = M11(sys.N_g,i)*sys.tpt.lambda_1 + M12_int(i);
        P12_plus(i) = M12(sys.N_g,i)*sys.tpt.lambda_2;
        P21_plus(i) = M21(sys.N_g,i)*sys.tpt.lambda_1 + T21(i) + M22_int(i);
        P22_plus(i) = M22(sys.N_g,i)*sys.tpt.lambda_2;
        P11_minus(i) = N11(sys.N_g,i)*sys.tpt.lambda_1 + N12_int(i);
        P12_minus(i) = N12(sys.N_g,i)*sys.tpt.lambda_2;
        P21_minus(i) = N21(sys.N_g,i)*sys.tpt.lambda_1 + N22_int(i);
        P22_minus(i) = N22(sys.N_g,i)*sys.tpt.lambda_2;
    end
end
