function [K] = solver_advection(params, N_g)

% Solves the one-dimensional linear advection equation
% \eps K_x(x, y) + K_y(x, y) = 0
% with boundary conditions
% K(x, 0) = f_1(x)
% K(0, y) = f_2(y)
% and 
% \eps > 0 
% over the square domain
% 0 \leq x, y \leq 1
% using an upwind scheme
% params gives:
% eps, f_1(x), f_2(y)

Delta = 1/(N_g - 1);

numTot = N_g^2; % Number of nodes to solve for

div_horizontal = params.eps/Delta;
div_vertical = 1/Delta;

div = div_horizontal + div_vertical;

div_horizontal_div = div_horizontal/div;
div_vertical_div = div_vertical/div;

% Want to put system in form Ax = b
% Definition of A matrix
vertical_block = zeros(N_g);
horizontal_block = eye(N_g);

A_vertical_small = zeros(N_g-1);
% Definition of B vector
b_horizontal = zeros(numTot, 1);
b_vertical = zeros(numTot, 1);

b = zeros(numTot, 1);

b_vertical(1) = params.f_1(1);
for i=2:N_g
    if(i < (N_g-1))
        horizontal_block(i+1, i) = -div_horizontal_div;
        A_vertical_small(i+1, i) = 1;
    end;
    vertical_block(i,i) = -div_vertical_div;
    b_horizontal(N_g*(i-1) + 1) = params.f_2(i);
    b_horizontal((N_g*(i-1) + 2)) = div_horizontal_div*params.f_2(i);
    b_vertical(i) = params.f_1(i);
    b_vertical(N_g + i) = div_vertical_div*params.f_1(i);
end;

A_horizontal = kron(eye(N_g - 1), horizontal_block);
A_vertical = kron(A_vertical_small, vertical_block);

A_sub = A_horizontal + A_vertical;
A = zeros(numTot);
A(1:N_g, 1:N_g) = eye(N_g);
A(N_g+1:numTot, N_g+1:numTot) = A_sub;

b = b_horizontal + b_vertical;

x = A \ b;

K = zeros(N_g);
glob_to_mat = [1:numTot];
K(glob_to_mat) = x;
K = K';



