function [sys, obss, obsb] = sim_loop_u_v(sys, obss, obsb)

% Initialize system storage variables
sys.ttt.u = zeros(sys.N_sim, sys.N_g_sim);
sys.ttt.v = zeros(sys.N_sim, sys.N_g_sim);
sys.ttt.un = sys.ttt.u0;
sys.ttt.vn = sys.ttt.v0;
sys.ttt.un1 = zeros(1, sys.N_g_sim);
sys.ttt.vn1 = zeros(1, sys.N_g_sim);
sys.ttt.u_norm = zeros(1, sys.N_sim);
sys.ttt.v_norm = zeros(1, sys.N_sim);
% Initialize bilateral observer storage variables

obsb.u = zeros(sys.N_sim, sys.N_g_sim);
obsb.v = zeros(sys.N_sim, sys.N_g_sim);
obsb.un1 = zeros(1, sys.N_g_sim);
obsb.vn1 = zeros(1, sys.N_g_sim);
obsb.y1 = 0;
obsb.y2 = 0;
obsb.m_error1 = 0;
obsb.m_error2 = 0;
obsb.u_error = zeros(sys.N_sim, sys.N_g_sim);
obsb.v_error = zeros(sys.N_sim, sys.N_g_sim);
obsb.un_error = zeros(1, sys.N_g_sim);
obsb.vn_error = zeros(1, sys.N_g_sim);
obsb.un_error_squared = 0;
obsb.vn_error_squared = 0;
obsb.u_error_norm = zeros(1, sys.N_sim);
obsb.v_error_norm = zeros(1, sys.N_sim);

% Initialize unilateral observer storage variables
obss.u = zeros(sys.N_sim, sys.N_g_sim);
obss.v = zeros(sys.N_sim, sys.N_g_sim);
obss.un1 = zeros(1, sys.N_g_sim);
obss.vn1 = zeros(1, sys.N_g_sim);
obss.y = 0;
obss.m_error = 0;
obss.u_error = zeros(sys.N_sim, sys.N_g_sim);
obss.v_error = zeros(sys.N_sim, sys.N_g_sim);
obss.un_error = zeros(1, sys.N_g_sim);
obss.vn_error = zeros(1, sys.N_g_sim);
obss.un_error_squared = 0;
obss.vn_error_squared = 0;
obss.u_error_norm = zeros(1, sys.N_sim);
obss.v_error_norm = zeros(1, sys.N_sim);

counter = 0;
for time=sys.Tspan
    counter = counter + 1;
    %% Bilateral observer
    %% Calculate bilateral observer estimation error
    obsb.un_error = sys.ttt.un - obsb.un;
    obsb.vn_error = sys.ttt.vn - obsb.vn;
    
    obsb.u_error(counter, :) = obsb.un_error;
    obsb.v_error(counter, :) = obsb.vn_error;
    
    obsb.un_error_squared = obsb.un_error.^2;
    obsb.vn_error_squared = obsb.vn_error.^2;
    
    obsb.u_error_norm(counter) = sqrt(trapz(sys.xspan_sim, obsb.un_error_squared));
    obsb.v_error_norm(counter) = sqrt(trapz(sys.xspan_sim, obsb.vn_error_squared));
    
    %% Make bilateral measurements and calculate u and v gains
    obsb.y1 = sys.ttt.un(sys.N_g_sim);
    obsb.y2 = sys.ttt.vn(1);
    
    obsb.m_error1 = obsb.y1 - obsb.un(sys.N_g_sim);
    obsb.m_error2 = obsb.y2 - obsb.vn(1);
    
    obsb.u_gain(1:(sys.N_g_sim-1)) = obsb.m_error1*obsb.P_pp(2:sys.N_g_sim) + obsb.m_error2*obsb.P_pm(2:sys.N_g_sim);
    obsb.v_gain(1:(sys.N_g_sim-1)) = obsb.m_error1*obsb.P_mp(1:(sys.N_g_sim-1)) + obsb.m_error2*obsb.P_mm(1:(sys.N_g_sim-1));
    
    %% Update bilateral observer
    obsb.u(counter, :) = obsb.un;
    obsb.v(counter, :) = obsb.vn;
    
    % Update points in domain
    obsb.du(1:(sys.N_g_sim-1)) = sys.ttt.sigma_p(2:sys.N_g_sim).*obsb.vn(2:sys.N_g_sim) - (sys.ttt.lambda/sys.Delta_sim)*(obsb.un(2:sys.N_g_sim) - obsb.un(1:(sys.N_g_sim-1)));
    obsb.dv(1:(sys.N_g_sim-1)) = sys.ttt.sigma_m(1:(sys.N_g_sim-1)).*obsb.un(1:(sys.N_g_sim-1)) + (sys.ttt.mu/sys.Delta_sim)*(obsb.vn(2:sys.N_g_sim) - obsb.vn(1:(sys.N_g_sim-1)));
    
    obsb.un1(2:sys.N_g_sim) = obsb.un(2:sys.N_g_sim) + sys.h*(obsb.du(1:(sys.N_g_sim-1)) + obsb.u_gain(1:(sys.N_g_sim-1)));
    obsb.vn1(1:(sys.N_g_sim-1)) = obsb.vn(1:(sys.N_g_sim-1)) + sys.h*(obsb.dv(1:(sys.N_g_sim-1)) + obsb.v_gain(1:(sys.N_g_sim-1)));
    
    % Update boundaries
    obsb.un1(1) = sys.ttt.Q*obsb.y2 + sys.ttt.U1(time) + sys.ttt.Theta(time-sys.ttt.distDelay);
    obsb.vn1(sys.N_g_sim) = sys.ttt.R*obsb.y1 + sys.ttt.U2(time);
    
    % Overwrite temporary state vectors
    obsb.un = obsb.un1;
    obsb.vn = obsb.vn1;
    
    %% Unilateral observer
    %% Calculate unilateral observer estimation error
    obss.un_error = sys.ttt.un - obss.un;
    obss.vn_error = sys.ttt.vn - obss.vn;
    
    obss.u_error(counter, :) = obss.un_error;
    obss.v_error(counter, :) = obss.vn_error;
    
    obss.un_error_squared = obss.un_error.^2;
    obss.vn_error_squared = obss.vn_error.^2;
    
    obss.u_error_norm(counter) = sqrt(trapz(sys.xspan_sim, obss.un_error_squared));
    obss.v_error_norm(counter) = sqrt(trapz(sys.xspan_sim, obss.vn_error_squared));
    
    %% Make right boundary measurement and calculate u and v gains
    obss.y = sys.ttt.un(sys.N_g_sim);
    
    obss.m_error = obss.y - obss.un(sys.N_g_sim);
    
    obss.u_gain(1:(sys.N_g_sim-1)) = obss.m_error*obss.P_p(2:sys.N_g_sim);
    obss.v_gain(1:(sys.N_g_sim-1)) = obss.m_error*obss.P_m(1:(sys.N_g_sim-1));
    
    %% Update right boundary observer
    obss.u(counter, :) = obss.un;
    obss.v(counter, :) = obss.vn;
    
    % Update points in domain
    obss.du(1:(sys.N_g_sim-1)) = sys.ttt.sigma_p(2:sys.N_g_sim).*obss.vn(2:sys.N_g_sim) - (sys.ttt.lambda/sys.Delta_sim)*(obss.un(2:sys.N_g_sim) - obss.un(1:(sys.N_g_sim-1)));
    obss.dv(1:(sys.N_g_sim-1)) = sys.ttt.sigma_m(1:(sys.N_g_sim-1)).*obss.un(1:(sys.N_g_sim-1)) + (sys.ttt.mu/sys.Delta_sim)*(obss.vn(2:sys.N_g_sim) - obss.vn(1:(sys.N_g_sim-1)));
    
    obss.un1(2:sys.N_g_sim) = obss.un(2:sys.N_g_sim) + sys.h*(obss.du(1:(sys.N_g_sim-1)) + obss.u_gain(1:(sys.N_g_sim-1)));
    obss.vn1(1:(sys.N_g_sim-1)) = obss.vn(1:(sys.N_g_sim-1)) + sys.h*(obss.dv(1:(sys.N_g_sim-1)) + obss.v_gain(1:(sys.N_g_sim-1)));
    
    % Update boundaries
    obss.un1(1) = sys.ttt.Q*obss.vn1(1) + sys.ttt.U1(time) + sys.ttt.Theta(time - sys.ttt.distDelay);
    obss.vn1(sys.N_g_sim) = sys.ttt.R*obss.y + sys.ttt.U2(time);
    
    % Overwrite temporary state vectors
    obss.un = obss.un1;
    obss.vn = obss.vn1;
    
    %% System dynamics
    %% Update system dynamics
    sys.ttt.u(counter, :) = sys.ttt.un;
    sys.ttt.v(counter, :) = sys.ttt.vn;
    
    sys.ttt.un_squared = sys.ttt.un.^2;
    sys.ttt.vn_squared = sys.ttt.vn.^2;
    
    sys.ttt.u_norm(counter) = sqrt(trapz(sys.xspan_sim, sys.ttt.un_squared));
    sys.ttt.v_norm(counter) = sqrt(trapz(sys.xspan_sim, sys.ttt.vn_squared));
    
    % Update points in domain
    sys.ttt.du(1:(sys.N_g_sim-1)) = sys.ttt.sigma_p(2:sys.N_g_sim).*sys.ttt.vn(2:sys.N_g_sim) - (sys.ttt.lambda/sys.Delta_sim)*(sys.ttt.un(2:sys.N_g_sim) - sys.ttt.un(1:(sys.N_g_sim-1)));
    sys.ttt.dv(1:(sys.N_g_sim-1)) = sys.ttt.sigma_m(1:(sys.N_g_sim-1)).*sys.ttt.un(1:(sys.N_g_sim-1)) + (sys.ttt.mu/sys.Delta_sim)*(sys.ttt.vn(2:sys.N_g_sim) - sys.ttt.vn(1:(sys.N_g_sim-1)));
    sys.ttt.un1(2:sys.N_g_sim) = sys.ttt.un(2:sys.N_g_sim) + sys.h*sys.ttt.du;
    sys.ttt.vn1(1:(sys.N_g_sim-1)) = sys.ttt.vn(1:(sys.N_g_sim-1)) + sys.h*sys.ttt.dv;
    
    % Update boundaries
    sys.ttt.un1(1) = sys.ttt.Q*sys.ttt.vn1(1) + sys.ttt.U1(time) + sys.ttt.Theta(time);
    sys.ttt.vn1(sys.N_g_sim) = sys.ttt.R*sys.ttt.un1(sys.N_g_sim) + sys.ttt.U2(time);
    
    % Overwrite temporary state vectors
    sys.ttt.un = sys.ttt.un1;
    sys.ttt.vn = sys.ttt.vn1;
end;
