%% Toy simulation 2(\lambda < \mu)
close all;
clear all;
warning('off','all');
str = fileparts(which('toy_case_2.m'));
cd(str);
run '../path_config.m'

%% User interface
% Instructions:
% Set flag = 1 to plot the respective quantity
% Set flag = 0 to not plot the respective quantity

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
plt.flags = [];                                   
plt.flags(toy_plot_enums.ttt_state_u) =                                         1;% 2X2 PDE system state u(x,t)
plt.flags(toy_plot_enums.ttt_state_v) =                                         1;% 2X2 PDE system state v(x,t) 
plt.flags(toy_plot_enums.bilat_obs_est_hat_u) =                                 0;% Bilateral observer estimate \hat{u}(x,t)
plt.flags(toy_plot_enums.bilat_obs_est_hat_v) =                                 0;% Bilateral observer estimate \hat{v}(x,t)
plt.flags(toy_plot_enums.unilat_obs_est_hat_u) =                                0;% Unilateral observer estimate \hat{u}(x,t)
plt.flags(toy_plot_enums.unilat_obs_est_hat_v) =                                0;% Unilateral observer estimate \hat{v}(x,t)
plt.flags(toy_plot_enums.bilat_obs_est_err_tild_u) =                            0;% Bilateral observer estimate error \tilde{u}(x,t)
plt.flags(toy_plot_enums.bilat_obs_est_err_tild_v) =                            0;% Bilateral observer estimate error \tilde{v}(x,t)
plt.flags(toy_plot_enums.unilat_obs_est_err_tild_u) =                           0;% Unilateral observer estimate error \tilde{u}(x,t)
plt.flags(toy_plot_enums.unilat_obs_est_err_tild_v) =                           0;% Unilateral observer estimate error \tilde{v}(x,t)
plt.flags(toy_plot_enums.obs_err_norm_u) =                                      0;% Observer Euclidean error norms ||u||(t)
plt.flags(toy_plot_enums.obs_err_norm_v) =                                      0;% Observer Eucludean error norms ||v||(t)
plt.flags(toy_plot_enums.bilat_obs_gain_P_pp) =                                 0;% Bilateral observer gain P^{++}(x)
plt.flags(toy_plot_enums.bilat_obs_gain_P_pm) =                                 0;% Bilateral observer gain P^{+-}(x)
plt.flags(toy_plot_enums.bilat_obs_gain_P_mp) =                                 0;% Bilateral observer gain P^{-+}(x)
plt.flags(toy_plot_enums.bilat_obs_gain_P_mm) =                                 0;% Bilateral observer gain P^{--}(x)
plt.flags(toy_plot_enums.unilat_obs_gain_P_p) =                                 0;% Unilateral observer gain P^+(x)
plt.flags(toy_plot_enums.unilat_obs_gain_P_m) =                                 0;% Unilateral observer gain P^-(x)
plt.flags(toy_plot_enums.kernel_M11) =                                          0;% M11 kernel equation solution
plt.flags(toy_plot_enums.kernel_M12) =                                          0;% M12 kernel equation solution
plt.flags(toy_plot_enums.kernel_M21) =                                          0;% M21 kernel equation solution
plt.flags(toy_plot_enums.kernel_M22) =                                          0;% M22 kernel equation solution
plt.flags(toy_plot_enums.kernel_N11) =                                          0;% N11 kernel equation solution
plt.flags(toy_plot_enums.kernel_N12) =                                          0;% N12 kernel equation solution
plt.flags(toy_plot_enums.kernel_N21) =                                          0;% N21 kernel equation solution
plt.flags(toy_plot_enums.kernel_N22) =                                          0;% N22 kernel equation solution
plt.flags(toy_plot_enums.kernel_K21) =                                          0;% K21 kernel equation solution
plt.flags(toy_plot_enums.kernel_M) =                                            0;% M kernel equation solution
plt.flags(toy_plot_enums.kernel_N) =                                            0;% N kernel equation solution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Simulation parameters
sys.N_g = 120;
sys.xspan = linspace(0, 1, sys.N_g);
sys.N_g_sim = 2*sys.N_g;
sys.xspan_sim = linspace(0, 1, sys.N_g_sim);
sys.Delta_sim = 1/(sys.N_g_sim - 1);
sys.simTime = 4;
sys.h = 0.001;
sys.Tspan = 0:sys.h:sys.simTime;
sys.N_sim = sys.simTime/sys.h + 1;

%% Plotting options
set(groot, 'DefaultTextInterpreter', 'Latex')
set(groot, 'DefaultLegendInterpreter', 'Latex')
sys.x_skip = 1;%Number of space domain grid points skipped per plotted spatial point
sys.t_skip = 5;%Number of time domain grid points skipped per plotted temporal point
plt.surf_view_time = [120, 40];
plt.surf_view_kernel = [-65, 20];

plt.axis_set_u_v_surf =  [0 1 0 sys.simTime -4 4];
plt.axis_set_u_v_plot =  [0 sys.simTime 0 1];
plt.axis_set_obs_gains = [0 1 -2 1];
plt.axis_set_kernel = [0 1 0 1 -1.5 1.5];

%Positions of plot |x|y|
plt.positions = []; 
plt.graphics = groot;
plt.x0_right = plt.graphics.MonitorPositions(1,1);
plt.y0_right = plt.graphics.MonitorPositions(1,2);
if(size(plt.graphics.MonitorPositions, 1) == 1) %1 monitor
    plt.x0_left = plt.x0_right;
    plt.y0_left = plt.y0_right;
else                                            %2 or more monitors
    plt.x0_left = plt.graphics.MonitorPositions(2,1);
    plt.y0_left = plt.graphics.MonitorPositions(2,2);
end
plt.screen_width = plt.graphics.ScreenSize(3);
plt.screen_height = plt.graphics.ScreenSize(4);

plt.positions(toy_plot_enums.ttt_state_u, :) =               [plt.x0_left,                           plt.y0_left + (2/3)*plt.screen_height ];% 2X2 PDE system state u(x,t)
plt.positions(toy_plot_enums.ttt_state_v, :) =               [plt.x0_left + (1/4)*plt.screen_width,  plt.y0_left + (2/3)*plt.screen_height ];% 2X2 PDE system state v(x,t) 
plt.positions(toy_plot_enums.bilat_obs_est_hat_u, :) =       [plt.x0_left,                           plt.y0_left + (1/3)*plt.screen_height ];% Bilateral observer estimate \hat{u}(x,t)
plt.positions(toy_plot_enums.bilat_obs_est_hat_v, :) =       [plt.x0_left,                           plt.y0_left                           ];% Bilateral observer estimate \hat{v}(x,t)
plt.positions(toy_plot_enums.unilat_obs_est_hat_u, :) =      [plt.x0_left + (1/4)*plt.screen_width,  plt.y0_left + (1/3)*plt.screen_height ];% Unilateral observer estimate \hat{u}(x,t)
plt.positions(toy_plot_enums.unilat_obs_est_hat_v, :) =      [plt.x0_left + (1/4)*plt.screen_width,  plt.y0_left                           ];% Unilateral observer estimate \hat{v}(x,t)
plt.positions(toy_plot_enums.bilat_obs_est_err_tild_u, :) =  [plt.x0_left + (2/4)*plt.screen_width,  plt.y0_left + (1/3)*plt.screen_height ];% Bilateral observer estimate error \tilde{u}(x,t)
plt.positions(toy_plot_enums.bilat_obs_est_err_tild_v, :) =  [plt.x0_left + (2/4)*plt.screen_width,  plt.y0_left                           ];% Bilateral observer estimate error \tilde{v}(x,t)
plt.positions(toy_plot_enums.unilat_obs_est_err_tild_u, :) = [plt.x0_left + (3/4)*plt.screen_width,  plt.y0_left + (1/3)*plt.screen_height ];% Unilateral observer estimate error \tilde{u}(x,t)
plt.positions(toy_plot_enums.unilat_obs_est_err_tild_v, :) = [plt.x0_left + (3/4)*plt.screen_width,  plt.y0_left                           ];% Unilateral observer estimate error \tilde{v}(x,t)
plt.positions(toy_plot_enums.obs_err_norm_u, :) =            [plt.x0_left + (2/4)*plt.screen_width,  plt.y0_left + (2/3)*plt.screen_height ];% Observer Euclidean error norms ||u||(t)
plt.positions(toy_plot_enums.obs_err_norm_v, :) =            [plt.x0_left + (3/4)*plt.screen_width,  plt.y0_left + (2/3)*plt.screen_height ];% Observer Eucludean error norms ||v||(t)
plt.positions(toy_plot_enums.bilat_obs_gain_P_pp, :) =       [plt.x0_right,                          plt.y0_right                          ];% Bilateral observer gain P^{++}(x)
plt.positions(toy_plot_enums.bilat_obs_gain_P_pm, :) =       [plt.x0_right + (1/5)*plt.screen_width, plt.y0_right                          ];% Bilateral observer gain P^{+-}(x)
plt.positions(toy_plot_enums.bilat_obs_gain_P_mp, :) =       [plt.x0_right + (2/5)*plt.screen_width, plt.y0_right                          ];% Bilateral observer gain P^{-+}(x)
plt.positions(toy_plot_enums.bilat_obs_gain_P_mm, :) =       [plt.x0_right + (3/5)*plt.screen_width, plt.y0_right                          ];% Bilateral observer gain P^{--}(x)
plt.positions(toy_plot_enums.unilat_obs_gain_P_p, :) =       [plt.x0_right + (4/5)*plt.screen_width, plt.y0_right + (1/4)*plt.screen_height];% Unilateral observer gain P^+(x)
plt.positions(toy_plot_enums.unilat_obs_gain_P_m, :) =       [plt.x0_right + (4/5)*plt.screen_width, plt.y0_right                          ];% Unilateral observer gain P^-(x)
plt.positions(toy_plot_enums.kernel_M11, :) =                [plt.x0_right,                          plt.y0_right + (3/4)*plt.screen_height];% M11 kernel equation solution
plt.positions(toy_plot_enums.kernel_M12, :) =                [plt.x0_right + (1/5)*plt.screen_width, plt.y0_right + (3/4)*plt.screen_height];% M12 kernel equation solution
plt.positions(toy_plot_enums.kernel_M21, :) =                [plt.x0_right,                          plt.y0_right + (2/4)*plt.screen_height];% M21 kernel equation solution
plt.positions(toy_plot_enums.kernel_M22, :) =                [plt.x0_right + (1/5)*plt.screen_width, plt.y0_right + (2/4)*plt.screen_height];% M22 kernel equation solution
plt.positions(toy_plot_enums.kernel_N11, :) =                [plt.x0_right + (2/5)*plt.screen_width, plt.y0_right + (3/4)*plt.screen_height];% N11 kernel equation solution
plt.positions(toy_plot_enums.kernel_N12, :) =                [plt.x0_right + (3/5)*plt.screen_width, plt.y0_right + (3/4)*plt.screen_height];% N12 kernel equation solution
plt.positions(toy_plot_enums.kernel_N21, :) =                [plt.x0_right + (2/5)*plt.screen_width, plt.y0_right + (2/4)*plt.screen_height];% N21 kernel equation solution
plt.positions(toy_plot_enums.kernel_N22, :) =                [plt.x0_right + (3/5)*plt.screen_width, plt.y0_right + (2/4)*plt.screen_height];% N22 kernel equation solution
plt.positions(toy_plot_enums.kernel_K21, :) =                [plt.x0_right + (4/5)*plt.screen_width, plt.y0_right + (3/4)*plt.screen_height];% K21 kernel equation solution
plt.positions(toy_plot_enums.kernel_M, :)   =                [plt.x0_right,                          plt.y0_right + (1/4)*plt.screen_height];% M kernel equation solution
plt.positions(toy_plot_enums.kernel_N, :)   =                [plt.x0_right + (1/5)*plt.screen_width, plt.y0_right + (1/4)*plt.screen_height];% N kernel equation solution

%Width and height of plots |width|height|
plt.size = [];                                    
plt.size(toy_plot_enums.ttt_state_u, :) =               [(1/4)*plt.screen_width, (1/3)*plt.screen_height];% 2X2 PDE system state u(x,t)
plt.size(toy_plot_enums.ttt_state_v, :) =               [(1/4)*plt.screen_width, (1/3)*plt.screen_height];% 2X2 PDE system state v(x,t) 
plt.size(toy_plot_enums.bilat_obs_est_hat_u, :) =       [(1/4)*plt.screen_width, (1/3)*plt.screen_height];% Bilateral observer estimate \hat{u}(x,t)
plt.size(toy_plot_enums.bilat_obs_est_hat_v, :) =       [(1/4)*plt.screen_width, (1/3)*plt.screen_height];% Bilateral observer estimate \hat{v}(x,t)
plt.size(toy_plot_enums.unilat_obs_est_hat_u, :) =      [(1/4)*plt.screen_width, (1/3)*plt.screen_height];% Unilateral observer estimate \hat{u}(x,t)
plt.size(toy_plot_enums.unilat_obs_est_hat_v, :) =      [(1/4)*plt.screen_width, (1/3)*plt.screen_height];% Unilateral observer estimate \hat{v}(x,t)
plt.size(toy_plot_enums.bilat_obs_est_err_tild_u, :) =  [(1/4)*plt.screen_width, (1/3)*plt.screen_height];% Bilateral observer estimate error \tilde{u}(x,t)
plt.size(toy_plot_enums.bilat_obs_est_err_tild_v, :) =  [(1/4)*plt.screen_width, (1/3)*plt.screen_height];% Bilateral observer estimate error \tilde{v}(x,t)
plt.size(toy_plot_enums.unilat_obs_est_err_tild_u, :) = [(1/4)*plt.screen_width, (1/3)*plt.screen_height];% Unilateral observer estimate error \tilde{u}(x,t)
plt.size(toy_plot_enums.unilat_obs_est_err_tild_v, :) = [(1/4)*plt.screen_width, (1/3)*plt.screen_height];% Unilateral observer estimate error \tilde{v}(x,t)
plt.size(toy_plot_enums.obs_err_norm_u, :) =            [(1/4)*plt.screen_width, (1/3)*plt.screen_height];% Observer Euclidean error norms ||u||(t)
plt.size(toy_plot_enums.obs_err_norm_v, :) =            [(1/4)*plt.screen_width, (1/3)*plt.screen_height];% Observer Eucludean error norms ||v||(t)
plt.size(toy_plot_enums.bilat_obs_gain_P_pp, :) =       [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% Bilateral observer gain P^{++}(x)
plt.size(toy_plot_enums.bilat_obs_gain_P_pm, :) =       [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% Bilateral observer gain P^{+-}(x)
plt.size(toy_plot_enums.bilat_obs_gain_P_mp, :) =       [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% Bilateral observer gain P^{-+}(x)
plt.size(toy_plot_enums.bilat_obs_gain_P_mm, :) =       [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% Bilateral observer gain P^{--}(x)
plt.size(toy_plot_enums.unilat_obs_gain_P_p, :) =       [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% Unilateral observer gain P^+(x)
plt.size(toy_plot_enums.unilat_obs_gain_P_m, :) =       [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% Unilateral observer gain P^-(x)
plt.size(toy_plot_enums.kernel_M11, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% M11 kernel equation solution
plt.size(toy_plot_enums.kernel_M12, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% M12 kernel equation solution
plt.size(toy_plot_enums.kernel_M21, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% M21 kernel equation solution
plt.size(toy_plot_enums.kernel_M22, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% M22 kernel equation solution
plt.size(toy_plot_enums.kernel_N11, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% N11 kernel equation solution
plt.size(toy_plot_enums.kernel_N12, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% N12 kernel equation solution
plt.size(toy_plot_enums.kernel_N21, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% N21 kernel equation solution
plt.size(toy_plot_enums.kernel_N22, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% N22 kernel equation solution
plt.size(toy_plot_enums.kernel_K21, :) =                [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% K21 kernel equation solution
plt.size(toy_plot_enums.kernel_M, :) =                  [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% M kernel equation solution
plt.size(toy_plot_enums.kernel_N, :) =                  [(1/5)*plt.screen_width, (1/4)*plt.screen_height];% N kernel equation solution
%% System parameters

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2X2 system parameters(ttt = two times two)
% 
% u_t(x,t) + \lambda u_x(x,t) = \sigma_+(x)v(x,t)
% v_t(x,t) - \mu     v_x(x,t) = \sigma_-(x)u(x,t)
%
% with boundary conditions
%
% u(0,t) = Q v(0,t) + U1(t)
% v(1,t) = R u(1,t) + U2(t)
%
% and initial conditions
%
% u(x,0) = u0(x);
% v(x,0) = v0(x);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ttt.lambda = 0.5;
ttt.mu = 1;
ttt.sigma_p_handle = @(x)(-2*exp(-sin(x.^3)));% Definition of sigma_+(x)
ttt.sigma_m_handle = @(x)(-1 + x - x.^2);% Definition of sigma_-(x)
ttt.sigma_p(1:sys.N_g_sim) = ttt.sigma_p_handle(sys.xspan_sim);
ttt.sigma_m(1:sys.N_g_sim) = ttt.sigma_m_handle(sys.xspan_sim);
ttt.Q = 1.5;%Q same as q in report
ttt.R = 2;%R same as \rho in report
ttt.U1 = @(t)(0);
ttt.U2 = @(t)(0);
ttt.Theta = @(t)(0);%External disturbance entering at left boundary
ttt.distDelay = 0;%Time delay for observers to "estimate" disturbance
ttt.u0_handle = @(x)(0.2*exp(0.5-x.^2));
ttt.v0_handle = @(x)(-sin(6*x));
ttt.u0(1:sys.N_g_sim) = ttt.u0_handle(sys.xspan_sim);
ttt.v0(1:sys.N_g_sim) = ttt.v0_handle(sys.xspan_sim);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sys.ttt = ttt;
clear ttt;
%% Observer gain calculations
% Bilateral observer gains(obsb => bilateral observer)
[obsb.P_pp, obsb.P_pm, obsb.P_mp, obsb.P_mm, obsb.M11, obsb.M12, obsb.M21, obsb.M22, obsb.N11, obsb.N12, obsb.N21, obsb.N22, obsb.K21] = bilateral_2t2_obs_gain_case_3(sys);%Case III since \lambda < \mu, x_s = 0.5
obsb.conv_time = max([(1/(sys.ttt.mu)) (1/(sys.ttt.lambda))]);

% Right sided observer gains(obss => single-sided observer, y = u(1,t))
[obss.P_p, obss.P_m, obss.M, obss.N] = right_2t2_obs_gain(sys);
obss.conv_time = (1/(sys.ttt.mu)) + (1/(sys.ttt.lambda));
%% Simulation loop

% Initial conditions for bilateral observer
obsb.un = zeros(1, sys.N_g_sim);
obsb.vn = zeros(1, sys.N_g_sim);

% Initial conditions for right boundary observer
obss.un = zeros(1, sys.N_g_sim);
obss.vn = zeros(1, sys.N_g_sim);

[sys, obss, obsb] = sim_loop_u_v(sys, obss, obsb);
%% Plots preprocessing
% Reduce number of elements in M and N
M(1:sys.N_g, 1:sys.N_g) = obss.M(1:2:sys.N_g_sim, 1:2:sys.N_g_sim);
N(1:sys.N_g, 1:sys.N_g) = obss.N(1:2:sys.N_g_sim, 1:2:sys.N_g_sim);

%Remove all kernel terms outside triangular domains
 for j=1:sys.N_g
        for i=j+1:sys.N_g
            obsb.M11(j,i) = nan;
            obsb.M12(j,i) = nan;
            obsb.M21(j,i) = nan;
            obsb.M22(j,i) = nan;
            obsb.N11(j,i) = nan;
            obsb.N12(j,i) = nan;
            obsb.N21(j,i) = nan;
            obsb.N22(j,i) = nan;
            M(j,i) = nan;
            N(j,i) = nan;
        end
 end
 
 %% Plots
if(plt.flags(toy_plot_enums.ttt_state_u))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.ttt_state_u, 1), plt.positions(toy_plot_enums.ttt_state_u, 2), plt.size(toy_plot_enums.ttt_state_u, 1), plt.size(toy_plot_enums.ttt_state_u,2)]);
    s = surf(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(1:sys.t_skip:sys.N_sim), sys.ttt.u(1:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('State $u(x,t)$ of $2\times2$ system');
    xlabel('Position $x$');
    ylabel('Time $t$');
    zlabel('State $u$');
    axis(plt.axis_set_u_v_surf);
    view(plt.surf_view_time);
end

if(plt.flags(toy_plot_enums.ttt_state_v))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.ttt_state_v, 1), plt.positions(toy_plot_enums.ttt_state_v, 2), plt.size(toy_plot_enums.ttt_state_v, 1), plt.size(toy_plot_enums.ttt_state_v,2)]);
    s = surf(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(1:sys.t_skip:sys.N_sim), sys.ttt.v(1:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('State $v(x,t)$ of $2\times2$ system');
    xlabel('Position $x$');
    ylabel('Time $t$');
    zlabel('State $v$');
    axis(plt.axis_set_u_v_surf);
    view(plt.surf_view_time);
end

if(plt.flags(toy_plot_enums.bilat_obs_est_hat_u))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.bilat_obs_est_hat_u, 1), plt.positions(toy_plot_enums.bilat_obs_est_hat_u, 2), plt.size(toy_plot_enums.bilat_obs_est_hat_u, 1), plt.size(toy_plot_enums.bilat_obs_est_hat_u,2)]);
    s = surf(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(1:sys.t_skip:sys.N_sim), obsb.u(1:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Bilateral observer state estimate $\hat{u}(x,t)$');
    xlabel('Position $x$');
    ylabel('Time $t$');
    zlabel('State estimate $\hat{u}$');
    axis(plt.axis_set_u_v_surf);
    view(plt.surf_view_time);
end

if(plt.flags(toy_plot_enums.bilat_obs_est_hat_v))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.bilat_obs_est_hat_v, 1), plt.positions(toy_plot_enums.bilat_obs_est_hat_v, 2), plt.size(toy_plot_enums.bilat_obs_est_hat_v, 1), plt.size(toy_plot_enums.bilat_obs_est_hat_v,2)]);
    s = surf(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(1:sys.t_skip:sys.N_sim), obsb.v(1:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Bilateral observer state estimate $\hat{v}(x,t)$');
    xlabel('Position $x$');
    ylabel('Time $t$');
    zlabel('State estimate $\hat{v}$');
    axis(plt.axis_set_u_v_surf);
    view(plt.surf_view_time);
end

if(plt.flags(toy_plot_enums.unilat_obs_est_hat_u))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.unilat_obs_est_hat_u, 1), plt.positions(toy_plot_enums.unilat_obs_est_hat_u, 2), plt.size(toy_plot_enums.unilat_obs_est_hat_u, 1), plt.size(toy_plot_enums.unilat_obs_est_hat_u,2)]);
    s = surf(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(1:sys.t_skip:sys.N_sim), obss.u(1:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Unilateral observer state estimate $\hat{u}(x,t)$');
    xlabel('Position $x$');
    ylabel('Time $t$');
    zlabel('State estimate $\hat{u}$');
    axis(plt.axis_set_u_v_surf);
    view(plt.surf_view_time);
end

if(plt.flags(toy_plot_enums.unilat_obs_est_hat_v))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.unilat_obs_est_hat_v, 1), plt.positions(toy_plot_enums.unilat_obs_est_hat_v, 2), plt.size(toy_plot_enums.unilat_obs_est_hat_v, 1), plt.size(toy_plot_enums.unilat_obs_est_hat_v,2)]);
    s = surf(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(1:sys.t_skip:sys.N_sim), obss.v(1:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Unilateral observer state estimate $\hat{v}(x,t)$');
    xlabel('Position $x$');
    ylabel('Time $t$');
    zlabel('State estimate $\hat{v}$');
    axis(plt.axis_set_u_v_surf);
    view(plt.surf_view_time);
end

if(plt.flags(toy_plot_enums.bilat_obs_est_err_tild_u))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.bilat_obs_est_err_tild_u, 1), plt.positions(toy_plot_enums.bilat_obs_est_err_tild_u, 2), plt.size(toy_plot_enums.bilat_obs_est_err_tild_u, 1), plt.size(toy_plot_enums.bilat_obs_est_err_tild_u,2)]);
    s = surf(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(1:sys.t_skip:sys.N_sim), obsb.u_error(1:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Bilateral observer state estimate error $\tilde{u}(x,t)$');
    xlabel('Position $x$');
    ylabel('Time $t$');
    zlabel('State estimate error $\tilde{u}$');
    axis(plt.axis_set_u_v_surf);
    view(plt.surf_view_time);
end

if(plt.flags(toy_plot_enums.bilat_obs_est_err_tild_v))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.bilat_obs_est_err_tild_v, 1), plt.positions(toy_plot_enums.bilat_obs_est_err_tild_v, 2), plt.size(toy_plot_enums.bilat_obs_est_err_tild_v, 1), plt.size(toy_plot_enums.bilat_obs_est_err_tild_v,2)]);
    s = surf(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(1:sys.t_skip:sys.N_sim), obsb.v_error(1:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Bilateral observer state estimate error $\tilde{v}(x,t)$');
    xlabel('Position $x$');
    ylabel('Time $t$');
    zlabel('State estimate error $\tilde{v}$');
    axis(plt.axis_set_u_v_surf);
    view(plt.surf_view_time);
end

if(plt.flags(toy_plot_enums.unilat_obs_est_err_tild_u))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.unilat_obs_est_err_tild_u, 1), plt.positions(toy_plot_enums.unilat_obs_est_err_tild_u, 2), plt.size(toy_plot_enums.unilat_obs_est_err_tild_u, 1), plt.size(toy_plot_enums.unilat_obs_est_err_tild_u,2)]);
    s = surf(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(1:sys.t_skip:sys.N_sim), obss.u_error(1:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Unilateral observer state estimate error $\tilde{u}(x,t)$');
    xlabel('Position $x$');
    ylabel('Time $t$');
    zlabel('State estimate error $\tilde{u}$');
    axis(plt.axis_set_u_v_surf);
    view(plt.surf_view_time);
end

if(plt.flags(toy_plot_enums.unilat_obs_est_err_tild_v))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.unilat_obs_est_err_tild_v, 1), plt.positions(toy_plot_enums.unilat_obs_est_err_tild_v, 2), plt.size(toy_plot_enums.unilat_obs_est_err_tild_v, 1), plt.size(toy_plot_enums.unilat_obs_est_err_tild_v,2)]);
    s = surf(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), sys.Tspan(1:sys.t_skip:sys.N_sim), obss.v_error(1:sys.t_skip:sys.N_sim, 1:sys.x_skip:sys.N_g_sim));
    s.EdgeColor = 'none';
    colormap jet;
    title('Unilateral observer state estimate error $\tilde{v}(x,t)$');
    xlabel('Position $x$');
    ylabel('Time $t$');
    zlabel('State estimate error $\tilde{v}$');
    axis(plt.axis_set_u_v_surf);
    view(plt.surf_view_time);
end

if(plt.flags(toy_plot_enums.obs_err_norm_u))
    figure;
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.obs_err_norm_u, 1), plt.positions(toy_plot_enums.obs_err_norm_u, 2), plt.size(toy_plot_enums.obs_err_norm_u, 1), plt.size(toy_plot_enums.obs_err_norm_u,2)]);
    hold on;
    p_bilateral = plot(sys.Tspan(1:sys.t_skip:sys.N_sim), obsb.u_error_norm(1:sys.t_skip:sys.N_sim));
    p_bilateral.Color = 'blue';
    p_bilateral.LineWidth = 2;
    p_unilateral = plot(sys.Tspan(1:sys.t_skip:sys.N_sim), obss.u_error_norm(1:sys.t_skip:sys.N_sim));
    p_unilateral.Color = 'red';
    p_unilateral.LineWidth = 2;
    line([obsb.conv_time obsb.conv_time], [0 1], 'Color', 'blue', 'LineStyle', '--');
    line([obss.conv_time obss.conv_time], [0 1], 'Color', 'red', 'LineStyle', '--');
    title('Unilateral and bilateral observer $||\tilde{u}(t)||$');
    xlabel('Time $t$');
    ylabel('Norm of estimation error $||\tilde{u}||$');
    legend('Bilateral observer $||\tilde{u}(t)||$', 'Unilateral observer $||\tilde{u}(t)||$', 'Bilateral observer $t_F = \max\{\frac{1}{\lambda}, \frac{1}{\mu}\}$', 'Unilateral observer $t_F = \frac{1}{\lambda} + \frac{1}{\mu}$');
    axis(plt.axis_set_u_v_plot);
end

if(plt.flags(toy_plot_enums.obs_err_norm_v))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.obs_err_norm_v, 1), plt.positions(toy_plot_enums.obs_err_norm_v, 2), plt.size(toy_plot_enums.obs_err_norm_v, 1), plt.size(toy_plot_enums.obs_err_norm_v,2)]);
    hold on
    p_bilateral = plot(sys.Tspan(1:sys.t_skip:sys.N_sim), obsb.v_error_norm(1:sys.t_skip:sys.N_sim));
    p_bilateral.Color = 'blue';
    p_bilateral.LineWidth = 2;
    p_unilateral = plot(sys.Tspan(1:sys.t_skip:sys.N_sim), obss.v_error_norm(1:sys.t_skip:sys.N_sim));
    p_unilateral.Color = 'red';
    p_unilateral.LineWidth = 2;
    line([obsb.conv_time obsb.conv_time], [0 1], 'Color', 'blue', 'LineStyle', '--');
    line([obss.conv_time obss.conv_time], [0 1], 'Color', 'red', 'LineStyle', '--');
    title('Unilateral and bilateral observer $||\tilde{v}(t)||$');
    xlabel('Time $t$');
    ylabel('Norm of estimation error $||\tilde{v}||$');
    legend('Bilateral observer $||\tilde{v}(t)||$', 'Unilateral observer $||\tilde{v}(t)||$', 'Bilateral observer $t_F = \max\{\frac{1}{\lambda}, \frac{1}{\mu}\}$', 'Unilateral observer $t_F = \frac{1}{\lambda} + \frac{1}{\mu}$');
    axis(plt.axis_set_u_v_plot);
end

if(plt.flags(toy_plot_enums.bilat_obs_gain_P_pp))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.bilat_obs_gain_P_pp, 1), plt.positions(toy_plot_enums.bilat_obs_gain_P_pp, 2), plt.size(toy_plot_enums.bilat_obs_gain_P_pp, 1), plt.size(toy_plot_enums.bilat_obs_gain_P_pp,2)]);
    p = plot(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), obsb.P_pp(1:sys.x_skip:sys.N_g_sim));
    p.Color = [0 0.4 0.6];%'blue';
    p.LineWidth = 2;
    title('Bilateral observer gain $P^{++}(x)$');
    xlabel('Position $x$');
    ylabel('Observer gain $P^{++}$');
    axis(plt.axis_set_obs_gains);
end

if(plt.flags(toy_plot_enums.bilat_obs_gain_P_pm))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.bilat_obs_gain_P_pm, 1), plt.positions(toy_plot_enums.bilat_obs_gain_P_pm, 2), plt.size(toy_plot_enums.bilat_obs_gain_P_pm, 1), plt.size(toy_plot_enums.bilat_obs_gain_P_pm,2)]);
    p = plot(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), obsb.P_pm(1:sys.x_skip:sys.N_g_sim));
    p.Color = [0.6 0 0.4];%'red';
    p.LineWidth = 2;
    title('Bilateral observer gain $P^{+-}(x)$');
    xlabel('Position $x$');
    ylabel('Observer gain $P^{+-}$');
    axis(plt.axis_set_obs_gains);
end

if(plt.flags(toy_plot_enums.bilat_obs_gain_P_mp))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.bilat_obs_gain_P_mp, 1), plt.positions(toy_plot_enums.bilat_obs_gain_P_mp, 2), plt.size(toy_plot_enums.bilat_obs_gain_P_mp, 1), plt.size(toy_plot_enums.bilat_obs_gain_P_mp,2)]);
    p = plot(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), obsb.P_mp(1:sys.x_skip:sys.N_g_sim));
    p.Color = 'green';
    p.LineWidth = 2;
    title('Bilateral observer gain $P^{-+}(x)$');
    xlabel('Position $x$');
    ylabel('Observer gain $P^{-+}$');
    axis(plt.axis_set_obs_gains);
end

if(plt.flags(toy_plot_enums.bilat_obs_gain_P_mm))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.bilat_obs_gain_P_mm, 1), plt.positions(toy_plot_enums.bilat_obs_gain_P_mm, 2), plt.size(toy_plot_enums.bilat_obs_gain_P_mm, 1), plt.size(toy_plot_enums.bilat_obs_gain_P_mm,2)]);
    p = plot(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), obsb.P_mm(1:sys.x_skip:sys.N_g_sim));
    p.Color = 'yellow';
    p.LineWidth = 2;
    title('Bilateral observer gain $P^{--}(x)$');
    xlabel('Position $x$');
    ylabel('Observer gain $P^{--}$');
    axis(plt.axis_set_obs_gains);
end

if(plt.flags(toy_plot_enums.unilat_obs_gain_P_p))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.unilat_obs_gain_P_p, 1), plt.positions(toy_plot_enums.unilat_obs_gain_P_p, 2), plt.size(toy_plot_enums.unilat_obs_gain_P_p, 1), plt.size(toy_plot_enums.unilat_obs_gain_P_p,2)]);
    p = plot(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), obss.P_p(1:sys.x_skip:sys.N_g_sim));
    p.Color = 'magenta';
    p.LineWidth = 2;
    title('Unilateral observer gain $\bar{P}^+(x)$');
    xlabel('Position $x$');
    ylabel('Observer gain $\bar{P}^+$');
    axis(plt.axis_set_obs_gains);
end

if(plt.flags(toy_plot_enums.unilat_obs_gain_P_m))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.unilat_obs_gain_P_m, 1), plt.positions(toy_plot_enums.unilat_obs_gain_P_m, 2), plt.size(toy_plot_enums.unilat_obs_gain_P_m, 1), plt.size(toy_plot_enums.unilat_obs_gain_P_m,2)]);
    p = plot(sys.xspan_sim(1:sys.x_skip:sys.N_g_sim), obss.P_m(1:sys.x_skip:sys.N_g_sim));
    p.Color = 'cyan';
    p.LineWidth = 2;
    title('Unilateral observer gain $\bar{P}^-(x)$');
    xlabel('Position $x$');
    ylabel('Observer gain $\bar{P}^-$');
    axis(plt.axis_set_obs_gains);
end

if(plt.flags(toy_plot_enums.kernel_M11))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.kernel_M11, 1), plt.positions(toy_plot_enums.kernel_M11, 2), plt.size(toy_plot_enums.kernel_M11, 1), plt.size(toy_plot_enums.kernel_M11,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.M11(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('Bilateral observer kernel $M_{11}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $M_{11}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(toy_plot_enums.kernel_M12))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.kernel_M12, 1), plt.positions(toy_plot_enums.kernel_M12, 2), plt.size(toy_plot_enums.kernel_M12, 1), plt.size(toy_plot_enums.kernel_M12,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.M12(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('Bilateral observer kernel $M_{12}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $M_{12}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(toy_plot_enums.kernel_M21))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.kernel_M21, 1), plt.positions(toy_plot_enums.kernel_M21, 2), plt.size(toy_plot_enums.kernel_M21, 1), plt.size(toy_plot_enums.kernel_M21,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.M21(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('Bilateral observer kernel $M_{21}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $M_{21}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(toy_plot_enums.kernel_M22))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.kernel_M22, 1), plt.positions(toy_plot_enums.kernel_M22, 2), plt.size(toy_plot_enums.kernel_M22, 1), plt.size(toy_plot_enums.kernel_M22,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.M22(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('Bilateral observer kernel $M_{22}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $M_{22}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(toy_plot_enums.kernel_N11))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.kernel_N11, 1), plt.positions(toy_plot_enums.kernel_N11, 2), plt.size(toy_plot_enums.kernel_N11, 1), plt.size(toy_plot_enums.kernel_N11,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.N11(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('Bilateral observer kernel $N_{11}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $N_{11}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(toy_plot_enums.kernel_N12))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.kernel_N12, 1), plt.positions(toy_plot_enums.kernel_N12, 2), plt.size(toy_plot_enums.kernel_N12, 1), plt.size(toy_plot_enums.kernel_N12,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.N12(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('Bilateral observer kernel $N_{12}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $N_{12}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(toy_plot_enums.kernel_N21))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.kernel_N21, 1), plt.positions(toy_plot_enums.kernel_N21, 2), plt.size(toy_plot_enums.kernel_N21, 1), plt.size(toy_plot_enums.kernel_N21,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.N21(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('Bilateral observer kernel $N_{21}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $N_{21}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(toy_plot_enums.kernel_N22))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.kernel_N22, 1), plt.positions(toy_plot_enums.kernel_N22, 2), plt.size(toy_plot_enums.kernel_N22, 1), plt.size(toy_plot_enums.kernel_N22,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.N22(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('Bilateral observer kernel $N_{22}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $N_{22}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(toy_plot_enums.kernel_K21))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.kernel_K21, 1), plt.positions(toy_plot_enums.kernel_K21, 2), plt.size(toy_plot_enums.kernel_K21, 1), plt.size(toy_plot_enums.kernel_K21,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), obsb.K21(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('Bilateral observer kernel $K_{21}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $K_{21}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(toy_plot_enums.kernel_M))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.kernel_M, 1), plt.positions(toy_plot_enums.kernel_M, 2), plt.size(toy_plot_enums.kernel_M, 1), plt.size(toy_plot_enums.kernel_M,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), M(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('Unilateral observer kernel $\bar{M}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $\bar{M}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end

if(plt.flags(toy_plot_enums.kernel_N))
    figure
    set(gcf,'units','pixels','position',[plt.positions(toy_plot_enums.kernel_N, 1), plt.positions(toy_plot_enums.kernel_N, 2), plt.size(toy_plot_enums.kernel_N, 1), plt.size(toy_plot_enums.kernel_N,2)]);
    s = surf(sys.xspan(1:sys.x_skip:sys.N_g), sys.xspan(1:sys.x_skip:sys.N_g), N(1:sys.x_skip:sys.N_g, 1:sys.x_skip:sys.N_g));
    s.EdgeColor = 'none';
    colormap hsv;
    title('Unilateral observer kernel $\bar{N}(x,\xi)$');
    xlabel('$x$');
    ylabel('$\xi$');
    zlabel('Kernel value $\bar{N}$');
    axis(plt.axis_set_kernel);
    view(plt.surf_view_kernel);
end
